/*Copyright (C) 2010 Michal Richter (michalisek@gmail.com)

This file is part of michalisekSpell.

MichalisekSpell is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
															   option) any later version.

MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with MichalisekSpell; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "AddDiacriticOOVGueser.hpp"
#include "Token.hpp"
#include "StagePosibility.hpp"
#include "DecoderLetter.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "DecoderLetter.hpp"
#include "ErrorModelDiacriticsSimple.hpp"
#include "utils.hpp"

namespace ngramchecker {


		string AddDiacriticOOVGueser::ResolveWord(string word, bool isName)
		{
			vector<Token> tokens;
			for (uint32_t i = 0; i < word.length(); i++)
			{
				uint32_t letter_ID = GlobalVars::stringMapper->MapChar(word[i]);
				tokens.push_back(Token(letter_ID));
			}

			DecoderLetter decoder = DecoderLetter(decoder_order, GlobalVars::errorModelDiacriticsSimpleP, false, isName);

			vector<StagePosibilityP> spv = decoder.DecodeTokenizedSentence(tokens);

			string ret = "";

			for (uint32_t i = decoder_order - 1; i < spv.size() - 1; i++)
			{
				ret += spv[i]->ToString();
			}

			if (ret != word)
			{
				double orig_cost = decoder.WordCost(word);
				double sug_cost = decoder.WordCost(ret);

				double word_change_cost;

				if (isName)
					word_change_cost = word_change_name_cost;
				else
					word_change_cost = word_change_normal_cost;


				if (sug_cost > orig_cost)
				{
					cerr << "orig: " << GlobalVars::stringMapper->ConvertBackFromISO(word) << "~" << orig_cost << ", sug: " << GlobalVars::stringMapper->ConvertBackFromISO(ret) << "~" << sug_cost << endl;
				}
				//assert(sug_cost <= orig_cost);

				if (sug_cost + word_change_cost > orig_cost)
					ret = word;
			}

			//cerr << "resolving completed!" << endl;
			return ret;

		}

}
