/*

 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with MichalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "MyIncreasingArray.hpp"
#include "CompIncreasingArray.hpp"


namespace ngramchecker {


		uint32_t CompIncreasingArray::GetSize()
		{
			return num_values;
		}

		uint32_t CompIncreasingArray::GetValueAt(uint32_t i)
		{
			return mia_vec[i / size_of_parts]->GetValueAt(i % size_of_parts);
		}

		CompIA_First_Last_IndexPair CompIncreasingArray::GetFirstLastIndexPair(uint32_t i)
		{
			CompIA_First_Last_IndexPair ret;

			uint32_t first = GetValueAt(i);

			uint32_t last;
			if (i < GetSize() - 1)
				last = GetValueAt(i + 1) - 1;
			else
				last = last_val_last_index;

			if (last >= first)
			{
				ret = CompIA_First_Last_IndexPair(new pair<uint32_t, uint32_t>(first, last));
			} //otherwise return NULL pointer

			return ret;
		}

		void CompIncreasingArray::WriteToStream(ofstream &ofs)
		{
			assert(ofs.is_open());

			uint32_t num_parts = mia_vec.size();
			ofs.write((char*)&num_parts, sizeof(uint32_t));
			ofs.write((char*)&size_of_parts, sizeof(uint32_t));
			ofs.write((char*)&num_values, sizeof(uint32_t));
			ofs.write((char*)&last_val_last_index, sizeof(uint32_t));

			for (uint32_t i = 0; i < mia_vec.size(); i++)
			{
				mia_vec[i]->WriteToStream(ofs);
			}
		}

		CompIncreasingArray::CompIncreasingArray(ifstream &ifs)
		{
			assert(ifs.is_open());
			uint32_t num_parts;

			ifs.read((char*)&num_parts, sizeof(uint32_t));
			ifs.read((char*)&size_of_parts, sizeof(uint32_t));
			ifs.read((char*)&num_values, sizeof(uint32_t));
			ifs.read((char*)&last_val_last_index, sizeof(uint32_t));

			for (uint32_t i = 0; i < num_parts; i++)
			{
				mia_vec.push_back(boost::shared_ptr<MyIncreasingArray >(new MyIncreasingArray(ifs)));
			}

		}

		CompIncreasingArray::CompIncreasingArray(vector<uint32_t> &val, uint32_t _last_val_last_index)
		{
			num_values = val.size();
			size_of_parts = 200;//val.size() / 100;

			uint32_t first = 0;
			uint32_t last = min((int)size_of_parts - 1, (int)(val.size()) - 1);

			while (first < num_values)
			{
				vector<uint32_t> part;
				for (uint32_t i = first; i <= last; i++)
					part.push_back(val[i]);
				mia_vec.push_back(boost::shared_ptr<MyIncreasingArray>(new MyIncreasingArray(part)));
				first += size_of_parts;
				last += size_of_parts;
				if (last >= num_values)
					last = num_values - 1;
			}

			for (uint32_t i = 0; i < num_values; i++)
			{
				assert(val[i] == GetValueAt(i));
			}

			last_val_last_index = _last_val_last_index;
		}

}
