/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef DECODERBASE_HPP_
#define DECODERBASE_HPP_

#include "StdAfx.h"
#include "ViterbiState.hpp"
#include "Token.hpp"

namespace ngramchecker {


class StagePosibility;
SP_DEF(StagePosibility);

class TransitionCostComputation;
SP_DEF(TransitionCostComputation);

typedef unordered_set<boost::shared_ptr<ViterbiState>, ViterbiStateP_ihash, ViterbiStateP_iequal_to> Trellis_stage_set;;

typedef vector<boost::shared_ptr<ViterbiState> > Trellis_stage;
SP_DEF(Trellis_stage);
SP_DEF(Trellis_stage_set);

typedef boost::shared_ptr<vector<vector<boost::shared_ptr<StagePosibility> > > > StagePosibilitiesType;

struct Pair_StagePosibilityP_double_comparer : less<pair<uint32_t, double> >
{
	bool operator()(const pair<boost::shared_ptr<StagePosibility>, double> &val1, const pair<boost::shared_ptr<StagePosibility>, double> &val2)
	{
		return val1.second < val2.second;
	}
};

class DecoderBase
{

private:
	uint32_t viterbi_order;


	StagePosibilitiesType stage_posibilities;

	boost::shared_ptr<StagePosibility> sentence_start_stage_posibility;
	boost::shared_ptr<StagePosibility> sentence_end_stage_posibility;
	TransitionCostComputationP transition_cost_computation;


	virtual vector<vector<StagePosibilityP> > init_inner_stage_posibilities(vector<Token> &tokens) = 0;

	void init_posibilities(vector<Token> &tokens);



public:

	DecoderBase(uint32_t _viterbi_order, StagePosibilityP _sentence_start_sp, StagePosibilityP _sentence_end_sp, TransitionCostComputationP _transition_cc):
		viterbi_order(_viterbi_order), sentence_start_stage_posibility(_sentence_start_sp), sentence_end_stage_posibility(_sentence_end_sp),
		transition_cost_computation(_transition_cc) {}


	uint32_t GetViterbiOrder();

	TransitionCostComputationP GetTransitionCostComputation();

	vector<StagePosibilityP>  DecodeTokenizedSentence(vector<Token> &tokens);

	void DecodeTokenizedSentence_ReturnStagePosibilities(vector<Token> &tokens, vector<StagePosibilityP> &decoded_sequence, StagePosibilitiesType &_stage_posibilities);

	string DecodeSentence(string sentence);

};

SP_DEF(DecoderBase);

}

#endif /* DECODERBASE_HPP_ */
