/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "DecoderFormsOnly.hpp"
#include "GlobalVars.hpp"
#include "SimWordsFinder.hpp"
#include "StagePosibility.hpp"
#include "TransitionCostComputation.hpp"
#include "ErrorModel.hpp"

namespace ngramchecker {

		vector<vector<StagePosibilityP> > DecoderFormsOnly::init_inner_stage_posibilities(vector<Token> &tokens)
		{
			vector<vector<StagePosibilityP> > ret;

			if (tokens.size() == 0)
				return ret;

			for (uint32_t i = 0; i < tokens.size(); i++)
			{
				vector<StagePosibilityP> vec_stage_pos;

				string word_str = tokens[i].GetWordStringISO();

				pair<string, bool> cash_key = pair<string, bool>(word_str, (i == 0) );

				if (GlobalVars::st_pos_singlefactor_cash->IsCashed(cash_key))
				{
					vec_stage_pos = GlobalVars::st_pos_singlefactor_cash->GetCashedValue(cash_key);
				}
				else
				{
					Similar_Words_Map msw = simWordsFinder->Find(tokens[i], (i == 0) );

					for (Similar_Words_Map::iterator it = msw.begin(); it != msw.end(); it++)
					{
						uint32_t sim_word_id = it->first;
						string sim_word_str = *(it->second.first);
						bool is_original = simWordsFinder->errModel->StringsAreIdentical(word_str, sim_word_str);
						double err_model_cost = it->second.second;
						vec_stage_pos.push_back(StagePosibilityP(new StagePosibility_FormsOnly(sim_word_id, sim_word_str, is_original, err_model_cost)));
					}

					std::sort(vec_stage_pos.begin(), vec_stage_pos.end(), StagePosibility_sort_cost());

					GlobalVars::st_pos_singlefactor_cash->StoreValueForKey(cash_key, vec_stage_pos);
				}

				ret.push_back(vec_stage_pos);
			}


			return ret;

		}

		DecoderFormsOnly::DecoderFormsOnly(uint32_t _viterbi_order, SimWordsFinderP _simWordsFinder, double _form_weight):
			DecoderBase(_viterbi_order, StagePosibility_FormsOnly::SentenceStartSP(), StagePosibility_FormsOnly::SentenceEndSP(),
					    TransitionCostComputationP((TransitionCostComputation*)(new TransitionCostComputation_FormsOnly(_form_weight)))),
			simWordsFinder(_simWordsFinder)
		{}


}

