/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "DecoderLetter.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "ErrorModel.hpp"

namespace ngramchecker {

		vector<vector<StagePosibilityP> > DecoderLetter::init_inner_stage_posibilities(vector<Token> &tokens)
		{
			vector<vector<StagePosibilityP> > ret;

			for (uint32_t i = 0; i < tokens.size(); i++)
			{
				vector<StagePosibilityP> vecSP;

				pair<string, bool> cash_key = pair<string, bool>(tokens[i].GetWordStringISO(), false);

				if (GlobalVars::st_pos_letter_cash->IsCashed(cash_key))
				{
					vecSP = GlobalVars::st_pos_letter_cash->GetCashedValue(cash_key);
				}
				else
				{
					uint32_t token_letter = tokens[i].ID;



					for (uint32_t j = 0; j < GlobalVars::stringMapper->NumLetters(); j++)
					{
						ErrorModelOutput emo = errModel->SubstitutionCost(GlobalVars::stringMapper->InvMapChar(token_letter), GlobalVars::stringMapper->InvMapChar(j), ignore_case);
						if (emo.cost < ErrorModel::max_cost)
						{
							double em_cost = 0;//emo.cost;

							StagePosibilityP spp = StagePosibilityP(new StagePosibility_Letter(j, em_cost));
							vecSP.push_back(spp);
						}
					}

					if (vecSP.size() == 0)
					{
						StagePosibilityP spp = StagePosibilityP(new StagePosibility_Letter(token_letter, 0));
						vecSP.push_back(spp);
					}

					std::sort(vecSP.begin(), vecSP.end(), StagePosibility_sort_cost());

					GlobalVars::st_pos_letter_cash->StoreValueForKey(cash_key, vecSP);
				}

				/*if (vecSP.size() == 0)
					cerr << "Error: vecSP.size() == 0: " << token_letter << endl;
				assert(vecSP.size() > 0);*/

				ret.push_back(vecSP);
			}


			return ret;

		}

		double DecoderLetter::WordCost(string word)
		{
			vector<StagePosibilityP> spp;

			uint32_t vit_order = this->GetViterbiOrder();

			for (uint32_t i = 0; i < vit_order - 1; i++)
				spp.push_back(StagePosibility_Letter::SentenceStartSP());

			for (uint32_t i = 0; i < word.length(); i++)
			{
				spp.push_back(StagePosibilityP(new StagePosibility_Letter(GlobalVars::stringMapper->MapChar(word[i]), 0)));
			}

			spp.push_back(StagePosibility_Letter::SentenceEndSP());

			double ret = 0;

			TransitionCostComputationP tccp = this->GetTransitionCostComputation();

			for (uint32_t i = vit_order - 1; i < spp.size(); i++)
			{

				ret += tccp->ComputeTransitionCostSPSequence(spp, i - vit_order + 1, i);
			}

			return ret;
		}

}