/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef DECODERLETTER_HPP_
#define DECODERLETTER_HPP_

#include "StdAfx.h"
#include "DecoderBase.hpp"
#include "TransitionCostComputation.hpp"
#include "StagePosibility.hpp"

namespace ngramchecker {

	class ErrorModel;

	SP_DEF(ErrorModel);

	class LetterLMSettings {
	public:
		bool use_letter_lm;
		uint32_t decoder_order;
		double change_cost_normal;
		double change_cost_name;

		LetterLMSettings(bool _use_letter_lm, uint32_t _decoder_order, double _change_cost_normal, double _change_cost_name):
			use_letter_lm(_use_letter_lm), decoder_order(_decoder_order), change_cost_normal(_change_cost_normal), change_cost_name(_change_cost_name)
		{}

		static LetterLMSettings DontUseIt()
		{
			return LetterLMSettings(false, 3, 0, 0);
		}

	};

	class DecoderLetter : public DecoderBase
	{

	private:

		ErrorModelP errModel;
		bool ignore_case;

		virtual vector<vector<StagePosibilityP> > init_inner_stage_posibilities(vector<Token> &tokens);

	public:

		DecoderLetter(uint32_t _viterbi_order, ErrorModelP _errModel, bool _ignore_case, bool _word_is_name):
			DecoderBase(_viterbi_order, StagePosibility_Letter::SentenceStartSP(), StagePosibility_Letter::SentenceEndSP(),
					    TransitionCostComputationP((TransitionCostComputation*)(new TransitionCostComputation_Letter(_word_is_name)))),
			errModel(_errModel), ignore_case(_ignore_case)
		{}

		double WordCost(string word);

	};


}



#endif /* DECODERLETTER*/
