/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */
#include "DecoderMorphoAnal.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "Lexicon.hpp"
#include "MorphologyMapper.hpp"
#include "StagePosibility.hpp"
#include "TransitionCostComputation.hpp"

namespace ngramchecker {

		vector<vector<StagePosibilityP> > DecoderMorphoAnal::init_inner_stage_posibilities(vector<Token> &tokens)
		{
			vector<vector<StagePosibilityP> > ret;

			for (uint32_t i = 0; i < tokens.size(); i++)
			{
				string word_str = tokens[i].GetWordStringISO();

				if (word_str.length() == 1 && GlobalVars::stringMapper->CharIsPunctuation(word_str[0]))
				{
					uint32_t punctID = GlobalVars::morphologyMapper->PunctuationUniversalIndex_char(word_str[0]);

					vector<StagePosibilityP> vecSP;
					vecSP.push_back(StagePosibilityP(new StagePosibility_MorphoAnal(punctID, punctID, punctID, 0.0, 0.0, 0.0, 0.0)));
					ret.push_back(vecSP);
				}
				else if (GlobalVars::morphologyMapper->GetNumberTokenID() == tokens[i].ID)
				{
					vector<StagePosibilityP> vecSP;
					vecSP.push_back(StagePosibilityP(new StagePosibility_MorphoAnal(tokens[i].ID, tokens[i].ID, tokens[i].ID, 0.0, 0.0, 0.0, 0.0)));
					ret.push_back(vecSP);
				}
				else
				{
					vector<StagePosibilityP> vec_stage_pos;
					int id = GlobalVars::lexicon->GetWordID(word_str);

					if (id >= 0)
					{
						uint32_t formID = (uint32_t)id;
						vectorP_WordFactors vwf = GlobalVars::morphologyMapper->GetOutputArcsPerWord(formID, true);

						for (uint32_t j = 0; j < vwf->size(); j++)
						{
							WordFactors wf = vwf->operator [](j);
							vec_stage_pos.push_back(StagePosibilityP(new StagePosibility_MorphoAnal(wf.factors[0], wf.factors[1], wf.factors[2], wf.emmision_probs[1], wf.emmision_probs[2], lemma_weight, tag_weight)));
						}

					}

					if (i == 0 && GlobalVars::stringMapper->CharIsUpperCase(word_str[0])) //On the beginning of the sentence both lower case and upper case starting letter should be tried!
					{
						string lc_word_str = word_str;
						lc_word_str[0] = GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(word_str[0]);

						int id = GlobalVars::lexicon->GetWordID(lc_word_str);

						if (id >= 0)
						{
							uint32_t formID = (uint32_t)id;
							vectorP_WordFactors vwf = GlobalVars::morphologyMapper->GetOutputArcsPerWord(formID, true);

							for (uint32_t j = 0; j < vwf->size(); j++)
							{
								WordFactors wf = vwf->operator [](j);
								vec_stage_pos.push_back(StagePosibilityP(new StagePosibility_MorphoAnal(wf.factors[0], wf.factors[1], wf.factors[2], wf.emmision_probs[1], wf.emmision_probs[2], lemma_weight, tag_weight)));
							}

						}

					}

					std::sort(vec_stage_pos.begin(), vec_stage_pos.end(), StagePosibility_sort_cost());

					if (vec_stage_pos.size() == 0)
					{
						if (GlobalVars::stringMapper->CharIsUpperCase(word_str[0]))
						{
							vec_stage_pos.push_back(StagePosibilityP(new StagePosibility_MorphoAnal(GlobalVars::morphologyMapper->GetNameTokenID(),
									GlobalVars::morphologyMapper->GetNameTokenID(), GlobalVars::morphologyMapper->GetNameTokenID(), 0.0, 0.0, 0.0, 0.0)));
						}
						else
						{
							vec_stage_pos.push_back(StagePosibilityP(new StagePosibility_MorphoAnal(GlobalVars::morphologyMapper->GetUnkTokenID(),
									GlobalVars::morphologyMapper->GetUnkTokenID(), GlobalVars::morphologyMapper->GetUnkTokenID(), 0.0, 0.0, 0.0, 0.0)));
						}
					}

					ret.push_back(vec_stage_pos);
				}

			}

			return ret;

		}

		DecoderMorphoAnal::DecoderMorphoAnal(uint32_t _viterbi_order, double _lemma_weight, double _tag_weight):
			DecoderBase(_viterbi_order, StagePosibility_MorphoAnal::SentenceStartSP(), StagePosibility_MorphoAnal::SentenceEndSP(),
						TransitionCostComputationP(new TransitionCostComputation_MorphoAnal(_lemma_weight, _tag_weight))),
			lemma_weight(_lemma_weight), tag_weight(_tag_weight)
		{}

}