/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */
#include "DecoderMultiFactor.hpp"
#include "GlobalVars.hpp"
#include "MorphologyMapper.hpp"
#include "StagePosibility.hpp"
#include "TransitionCostComputation.hpp"
#include "SimWordsFinder.hpp"
#include "ErrorModel.hpp"

namespace ngramchecker {

		vector<vector<StagePosibilityP> > DecoderMultiFactor::init_inner_stage_posibilities(vector<Token> &tokens)
		{
			vector<vector<StagePosibilityP> > ret;

			if (tokens.size() == 0)
				return ret;

			/*vector<uint32_t> word_vec0_lc = tokens[0].GetWordVector();
			string word_str0_lc = tokens[0].GetWordStringISO();

			if (GlobalVars::stringMapper)*/

			for (uint32_t i = 0; i < tokens.size(); i++)
			{
				string word_str = tokens[i].GetWordStringISO();

				vector<StagePosibilityP> vec_stage_pos;

				if (GlobalVars::st_pos_multifactor_cash->IsCashed(pair<string, bool>(word_str, (i == 0)) ))
				{
					vec_stage_pos = GlobalVars::st_pos_multifactor_cash->GetCashedValue(pair<string, bool>(word_str, (i == 0)));
				}
				else
				{
					Similar_Words_Map msw = simWordsFinder->Find(tokens[i], (i == 0)/*is it sentence start?*/ );


					for (Similar_Words_Map::iterator it = msw.begin(); it != msw.end(); it++)
					{
						uint32_t sim_word_id = it->first;
						string sim_word_str = *(it->second.first);
						bool is_original = simWordsFinder->errModel->StringsAreIdentical(word_str, sim_word_str);
						double err_model_cost = it->second.second;

						vectorP_WordFactors vwf = GlobalVars::morphologyMapper->GetOutputArcsPerWord(sim_word_id, true);

						for (uint32_t j = 0; j < vwf->size(); j++)
						{
							WordFactors wf = vwf->operator [](j);

							StagePosibilityP spp = StagePosibility_MultiFactor::CreateSP_ComplexEmCost(wf.factors[0], wf.factors[1], wf.factors[2],
															sim_word_str, is_original, err_model_cost, wf.emmision_probs[1], wf.emmision_probs[2],
															lemma_model_weight, tag_model_weight);
							vec_stage_pos.push_back(spp);
						}

					}

					std::sort(vec_stage_pos.begin(), vec_stage_pos.end(), StagePosibility_sort_cost());

					GlobalVars::st_pos_multifactor_cash->StoreValueForKey(pair<string, bool>(word_str, (i == 0)), vec_stage_pos);
				}
				ret.push_back(vec_stage_pos);
			}


			return ret;

		}

		DecoderMultiFactor::DecoderMultiFactor(uint32_t form_lm_order, uint32_t lemma_lm_order, uint32_t tag_lm_order, SimWordsFinderP _simWordsFinder, double _form_model_weight, double _lemma_model_weight, double _tag_model_weight):
			DecoderBase(max(max(form_lm_order, lemma_lm_order), tag_lm_order), StagePosibility_MultiFactor::SentenceStartSP(), StagePosibility_MultiFactor::SentenceEndSP(),
					    TransitionCostComputationP(new TransitionCostComputation_MultiFactor(form_lm_order, lemma_lm_order, tag_lm_order, _form_model_weight, _lemma_model_weight, _tag_model_weight))),
			form_model_weight(_form_model_weight), lemma_model_weight(_lemma_model_weight), tag_model_weight(_tag_model_weight),
			simWordsFinder(_simWordsFinder)
		{}


}
