/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef ERRORMODEL_HPP_
#define ERRORMODEL_HPP_

#include "StdAfx.h"

namespace ngramchecker {

	class ErrorModelOutput {
	public:
		uint32_t edit_dist;
		double cost;

		ErrorModelOutput(uint32_t _edit_dist, double _cost): edit_dist(_edit_dist), cost(_cost) {}

		ErrorModelOutput(): edit_dist(0), cost(0.0) {}
	};

	class ErrorModel
	{
	public:
		static const double impossible_cost/* = 1000*/;

		static const double max_cost;

		virtual double UnknownWordCost() = 0;

		virtual double NameCost() = 0;

		virtual ErrorModelOutput SubstitutionCost(char char1, char letter2, bool ignore_case = false) = 0;

		virtual ErrorModelOutput InsertionCost(char inserted_char, char previous_char, char next_char) = 0;

		virtual ErrorModelOutput SwapCost(char first_char, char second_char) = 0;

		virtual ErrorModelOutput DeletionCost(char current_char, char previous_char, char next_char) = 0;

		virtual uint32_t MinOperationEditDistance() = 0;

		virtual bool StringsAreIdentical(const string &s1, const string &s2) = 0;

		virtual string ToString() = 0;

	};

	SP_DEF(ErrorModel);

	//ErrorModelTypoP errorModel;

}


#endif /* ERRORMODEL_HPP_ */
