/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */
#include "ErrorModelAbbreviation.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"

namespace ngramchecker {

	    double ErrorModelAbbreviation::UnknownWordCost()
		{
			return ErrorModel::impossible_cost;
		}

		double ErrorModelAbbreviation::NameCost()
		{
			return ErrorModel::impossible_cost;
		}

		ErrorModelOutput ErrorModelAbbreviation::SubstitutionCost(char char1, char char2, bool ignore_case)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		//TODO: remove parameters
		ErrorModelOutput ErrorModelAbbreviation::InsertionCost(char char_inserted, char previous_char, char next_char)
		{
				if (char_inserted == previous_char || char_inserted == next_char)
					return ErrorModelOutput(0, 0);
				else if (GlobalVars::stringMapper->CharIsVocal(char_inserted))
					return ErrorModelOutput(0, 0);
				else
					return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		ErrorModelOutput ErrorModelAbbreviation::SwapCost(char first_char, char second_char)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		ErrorModelOutput ErrorModelAbbreviation::DeletionCost(char current_char, char previous_char, char next_char)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		uint32_t ErrorModelAbbreviation::MinOperationEditDistance()
		{
			return 0;
		}

		bool ErrorModelAbbreviation::StringsAreIdentical(const string &s1, const string &s2)
		{
			return s1 == s2;
		}

		string ErrorModelAbbreviation::ToString()
		{
			return "abbreviation";
		}

}