/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "ErrorModelDiacriticsComplex.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"

namespace ngramchecker {

		double ErrorModelDiacriticsComplex::UnknownWordCost()
		{
			return ErrorModel::impossible_cost;
		}

		double ErrorModelDiacriticsComplex::NameCost()
		{
			return ErrorModel::impossible_cost;
		}

		ErrorModelOutput ErrorModelDiacriticsComplex::SubstitutionCost(char char1, char char2, bool ignore_case)
		{
			if (char1 == char2)
			{
				return ErrorModelOutput(0, 0.0);
			}

			if (ignore_case == false)
			{
				map<pair<char, char>, double>::iterator fit = substitutions.find(pair<char, char>(char1, char2));
				if (fit == substitutions.end())
				{
					return ErrorModelOutput(1, ErrorModel::impossible_cost);
				}
				else
				{
					return ErrorModelOutput(0, fit->second);
				}
			}
			else
			{
				if (GlobalVars::stringMapper->CharIsUpperCase(char1))
					char1 = GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(char1);

				if (GlobalVars::stringMapper->CharIsUpperCase(char2))
					char2 = GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(char2);

				return SubstitutionCost(char1, char2, false);
			}
		}

		ErrorModelOutput ErrorModelDiacriticsComplex::InsertionCost(char char_inserted, char previous_char, char next_char)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		ErrorModelOutput ErrorModelDiacriticsComplex::SwapCost(char first_char, char second_char)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		ErrorModelOutput ErrorModelDiacriticsComplex::DeletionCost(char current_char, char previous_char, char next_char)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		uint32_t ErrorModelDiacriticsComplex::MinOperationEditDistance()
		{
			return 0;
		}

		bool ErrorModelDiacriticsComplex::StringsAreIdentical(const string &s1, const string &s2)
		{
			return s1 == s2;
		}

		string ErrorModelDiacriticsComplex::ToString()
		{
			return "complex";
		}

		ErrorModelDiacriticsComplex::ErrorModelDiacriticsComplex()
		{
			ifstream ifs;
			ifs.open(MyConstants::filename_error_model_diacritics.c_str());
			assert(ifs.is_open());

			string s;
			while (std::getline(ifs, s))
			{
				vector<string> parts;

				boost::split(parts, s, boost::is_any_of(" "));

				string letters = parts[0];

				for (uint32_t i = 1; i < letters.length(); i++)
				{
					substitutions[pair<char, char>(letters[0], letters[i])] = boost::lexical_cast<double>(parts[i]);
				}
			}

		}

}