/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef ERRORMODELDIACRITICSCOMPLEX_HPP_
#define ERRORMODELDIACRITICSCOMPLEX_HPP_

#include "StdAfx.h"
#include "ErrorModel.hpp"

namespace ngramchecker {

	class ErrorModelDiacriticsComplex : public ErrorModel
	{
	private:

		map<pair<char, char>, double> substitutions;
	public:

		ErrorModelDiacriticsComplex();

		virtual double UnknownWordCost();

		virtual double NameCost();

		virtual ErrorModelOutput SubstitutionCost(char char1, char letter2, bool ignore_case = false);

		virtual ErrorModelOutput InsertionCost(char inserted_char, char previous_char, char next_char);

		virtual ErrorModelOutput SwapCost(char first_char, char second_char);

		virtual ErrorModelOutput DeletionCost(char current_char, char previous_char, char next_char);

		virtual uint32_t MinOperationEditDistance();

		virtual bool StringsAreIdentical(const string &s1, const string &s2);

		virtual string ToString();

	};

	SP_DEF(ErrorModelDiacriticsComplex);

}

#endif /* ERRORMODELDIACRITICSCOMPLEX_HPP_ */
