/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "ErrorModelDiacriticsSimple.hpp"
#include "GlobalVars.hpp"
#include "ErrorModelDiacriticsComplex.hpp"

namespace ngramchecker {


		double ErrorModelDiacriticsSimple::UnknownWordCost()
		{
			return ErrorModel::impossible_cost;
		}


		double ErrorModelDiacriticsSimple::NameCost()
		{
			return ErrorModel::impossible_cost;
		}

		ErrorModelOutput ErrorModelDiacriticsSimple::SubstitutionCost(char char1, char char2, bool ignore_case)
		{
			ErrorModelOutput emo = GlobalVars::errorModelDiacriticsComplexP->SubstitutionCost(char1, char2, ignore_case);

			if (emo.cost != ErrorModel::impossible_cost)
			{
				emo.cost = 0;
			}

			return emo;
		}

		ErrorModelOutput ErrorModelDiacriticsSimple::InsertionCost(char char_inserted, char previous_char, char next_char)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		ErrorModelOutput ErrorModelDiacriticsSimple::SwapCost(char first_char, char second_char)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		ErrorModelOutput ErrorModelDiacriticsSimple::DeletionCost(char current_char, char previous_char, char next_char)
		{
			return ErrorModelOutput(1, ErrorModel::impossible_cost);
		}

		uint32_t ErrorModelDiacriticsSimple::MinOperationEditDistance()
		{
			return 0;
		}

		bool ErrorModelDiacriticsSimple::StringsAreIdentical(const string &s1, const string &s2)
		{
			return s1 == s2;
		}

		string ErrorModelDiacriticsSimple::ToString()
		{
			return "simple";
		}
}
