/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "ErrorModelIgnoreDiacriticsTypo.hpp"
#include "GlobalVars.hpp"
#include "ErrorModelDiacriticsSimple.hpp"
#include "StringMapper.hpp"

namespace ngramchecker {

		double ErrorModelIgnoreDiacriticsTypo::UnknownWordCost()
		{
			return err_typo->UnknownWordCost();
		}


		double ErrorModelIgnoreDiacriticsTypo::NameCost()
		{
			return err_typo->NameCost();
		}

		ErrorModelOutput ErrorModelIgnoreDiacriticsTypo::SubstitutionCost(char char1, char char2, bool ignore_case)
		{
			assert(ignore_case == false);

			double val = GlobalVars::errorModelDiacriticsSimpleP->SubstitutionCost(char1, char2).cost;

			if (val <= 500)
			{
				return ErrorModelOutput(0, 0.0);
			}
			else
			{
				return err_typo->SubstitutionCost(char1, char2);
			}

		}

		ErrorModelOutput ErrorModelIgnoreDiacriticsTypo::InsertionCost(char char_inserted, char previous_char, char next_char)
		{
			return err_typo->InsertionCost(char_inserted, previous_char, next_char);
		}

		ErrorModelOutput ErrorModelIgnoreDiacriticsTypo::SwapCost(char first_char, char second_char)
		{
			return err_typo->SwapCost(first_char, second_char);
		}

		ErrorModelOutput ErrorModelIgnoreDiacriticsTypo::DeletionCost(char current_char, char previous_char, char next_char)
		{
			return err_typo->DeletionCost(current_char, previous_char, next_char);
		}

		uint32_t ErrorModelIgnoreDiacriticsTypo::MinOperationEditDistance()
		{
			return 0;
		}

		bool ErrorModelIgnoreDiacriticsTypo::StringsAreIdentical(const string &s1, const string &s2)
		{
			if (s1.length() != s2.length())
				return false;

			if (s1 == s2)
				return true;

			string s1_wd = GlobalVars::stringMapper->RemoveDiacritics_ISO(s1);
			string s2_wd = GlobalVars::stringMapper->RemoveDiacritics_ISO(s2);

			return s1_wd == s2_wd;
		}

		string ErrorModelIgnoreDiacriticsTypo::ToString()
		{
			return "IgnoreDiacriticsTypo";
		}
}
