/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef ERRORMODELTYPO_HPP_
#define ERRORMODELTYPO_HPP_

#include "StdAfx.h"
#include "ErrorModel.hpp"

namespace ngramchecker {

	typedef pair<char, char> char_pair;
	typedef set<pair<char, char> > set_pair;
	typedef map<char, vector<char> > map_char_vector;

	class ErrorModelTypo : public ErrorModel
	{

		double cost_sub_hadj;
		double cost_sub_random;
		double cost_sub_vadj;
		double cost_sub_zs; //z is error, s is true
		double cost_sub_vocal;
		double cost_sub_diac_del;
		double cost_sub_diac_add;
		double cost_sub_yi; //y is error, i is true
		double cost_sub_sz;  //s is error, z is true
		double cost_sub_iy; //i is error, y is true

		double cost_delete_random;
		double cost_delete_same;
		double cost_delete_hadj;
		double cost_delete_vadj;
		double cost_sub_capitalization;

		double cost_insert;

		double cost_swap;

		set_pair hadj_letters;
		set_pair vadj_letters;

		map_char_vector hadj_map;
		map_char_vector vadj_map;

		map_char_vector vocal_sub_map;
		map_char_vector add_diac_map;
		map<char, char> del_diac_map;

		map<char, vector<pair<char, double> > > special_substitutions_map;


		string lc_chars;
		string uc_chars;

		string lc_chars_no_diacritics;
		string uc_chars_no_diacritics;

		map<char_pair, double> substitution_cost_map;

	public:

		ErrorModelTypo(string error_model_file);
		void PrintCosts();

		virtual double UnknownWordCost();

		virtual double NameCost();

		virtual ErrorModelOutput SubstitutionCost(char char1, char letter2, bool ignore_case = false);

		virtual ErrorModelOutput InsertionCost(char inserted_char, char previous_char, char next_char);

		virtual ErrorModelOutput SwapCost(char first_char, char second_char);

		virtual ErrorModelOutput DeletionCost(char current_char, char previous_char, char next_char);

		virtual uint32_t MinOperationEditDistance();

		virtual bool StringsAreIdentical(const string &s1, const string &s2);

		virtual string ToString();

	};

	SP_DEF(ErrorModelTypo);

}

#endif /* ERRORMODELTYPO_HPP_ */
