/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "ErrorModelTypo_Dia_and_Cap_Zero_Dist.hpp"
#include "GlobalVars.hpp"
#include "ErrorModelTypo.hpp"
#include "ErrorModelDiacriticsSimple.hpp"
#include "StringMapper.hpp"

namespace ngramchecker {

		double ErrorModelTypo_Dia_and_Cap_Zero_Dist::UnknownWordCost()
		{
			return GlobalVars::errorModelTypoP->UnknownWordCost();
		}


		double ErrorModelTypo_Dia_and_Cap_Zero_Dist::NameCost()
		{
			return GlobalVars::errorModelTypoP->NameCost();
		}

		ErrorModelOutput ErrorModelTypo_Dia_and_Cap_Zero_Dist::SubstitutionCost(char char1, char char2, bool ignore_case)
		{
			if (GlobalVars::errorModelDiacriticsSimpleP->SubstitutionCost(char1, char2, ignore_case).edit_dist == 0)
			{
				return ErrorModelOutput(0, 0/*GlobalVars::errorModelTypoP->SubstitutionCost(char1, char2, ignore_case).cost*/);
			}
			else if ( (GlobalVars::stringMapper->CharIsUpperCase(char1) && char2 == GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(char1)) ||
					  (GlobalVars::stringMapper->CharIsUpperCase(char2) && char1 == GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(char2)))
			{
				return ErrorModelOutput(0, case_substitution_cost);
			}
			else
			{
				return GlobalVars::errorModelTypoP->SubstitutionCost(char1, char2, ignore_case);
			}
		}

		ErrorModelOutput ErrorModelTypo_Dia_and_Cap_Zero_Dist::InsertionCost(char inserted_char, char previous_char, char next_char)
		{
			return GlobalVars::errorModelTypoP->InsertionCost(inserted_char, previous_char, next_char);
		}

		ErrorModelOutput ErrorModelTypo_Dia_and_Cap_Zero_Dist::SwapCost(char first_char, char second_char)
		{
			return GlobalVars::errorModelTypoP->SwapCost(first_char, second_char);
		}

		ErrorModelOutput ErrorModelTypo_Dia_and_Cap_Zero_Dist::DeletionCost(char current_char, char previous_char, char next_char)
		{
			return GlobalVars::errorModelTypoP->DeletionCost(current_char, previous_char, next_char);
		}

		uint32_t ErrorModelTypo_Dia_and_Cap_Zero_Dist::MinOperationEditDistance()
		{
			return 0;
		}

		bool ErrorModelTypo_Dia_and_Cap_Zero_Dist::StringsAreIdentical(const string &s1, const string &s2)
		{
			return GlobalVars::errorModelTypoP->StringsAreIdentical(s1, s2);
		}

		string ErrorModelTypo_Dia_and_Cap_Zero_Dist::ToString()
		{
			return "typo: diacritic and capitalization substitution zero distance (but nonzero cost)";
		}

		const double ErrorModelTypo_Dia_and_Cap_Zero_Dist::case_substitution_cost = 1;

}