/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "MorphologyMapper.hpp"
#include "ErrorModelAbbreviation.hpp"
#include "ErrorModelDiacriticsComplex.hpp"
#include "ErrorModelDiacriticsSimple.hpp"
#include "ErrorModelIgnoreDiacriticsTypo.hpp"
#include "ErrorModelTypo_Dia_and_Cap_Zero_Dist.hpp"
#include "ErrorModelTypo.hpp"
#include "Lexicon.hpp"
#include "LemmaSigns.hpp"
#include "MyCash.hpp"
#include "LMServer.hpp"
#include "TextCheckingServer.hpp"

namespace ngramchecker {

		
	void GlobalVars::InitGlobalVars ()
	{
		GlobalVars::stringMapper = boost::shared_ptr<StringMapper>(new StringMapper());
		GlobalVars::morphologyMapper = boost::shared_ptr<MorphologyMapper>(new MorphologyMapper(true, MyConstants::default_morphology_cash_capacity));
		
		GlobalVars::errorModelDiacriticsSimpleP = boost::shared_ptr<ErrorModelDiacriticsSimple>(new ErrorModelDiacriticsSimple());
		GlobalVars::errorModelDiacriticsComplexP = boost::shared_ptr<ErrorModelDiacriticsComplex>(new ErrorModelDiacriticsComplex());
		GlobalVars::errorModelTypoP = boost::shared_ptr<ErrorModel>(new ErrorModelTypo(MyConstants::filename_error_model_type_auto));
		GlobalVars::errorModelTypoDiaCapZeroDistP = ErrorModelTypo_Dia_and_Cap_Zero_DistP(new ErrorModelTypo_Dia_and_Cap_Zero_Dist());
		GlobalVars::errorModelIgnoreDiacriticsTypoP = boost::shared_ptr<ErrorModelIgnoreDiacriticsTypo>(new ErrorModelIgnoreDiacriticsTypo(GlobalVars::errorModelTypoP));
		GlobalVars::errorModelIgnoreDiacriticsTypoDiaCapZeroDistP = boost::shared_ptr<ErrorModelIgnoreDiacriticsTypo>(new ErrorModelIgnoreDiacriticsTypo(GlobalVars::errorModelTypoDiaCapZeroDistP));
		
		GlobalVars::errorModelAbbreviationP = boost::shared_ptr<ErrorModelAbbreviation>(new ErrorModelAbbreviation());
		
		GlobalVars::lexicon = boost::shared_ptr<Lexicon>(Lexicon::ReadFromBinFile(MyConstants::filename_lexicon_bin));
		GlobalVars::lmServer = boost::shared_ptr<LMServer>(new LMServer());
		GlobalVars::textCheckingServer = boost::shared_ptr<TextCheckingServer>(new TextCheckingServer());
		GlobalVars::lemmaSigns = boost::shared_ptr<LemmaSigns>(new LemmaSigns());
		GlobalVars::st_pos_multifactor_cash = boost::shared_ptr<MyCash_StagePosibility>(new MyCash_StagePosibility(MyConstants::default_stage_posibility_cash_capacity));
		GlobalVars::st_pos_singlefactor_cash = boost::shared_ptr<MyCash_StagePosibility>(new MyCash_StagePosibility(MyConstants::default_stage_posibility_cash_capacity));
		GlobalVars::st_pos_letter_cash = boost::shared_ptr<MyCash_StagePosibility>(new MyCash_StagePosibility(MyConstants::default_stage_posibility_cash_capacity));
		
	}

	boost::shared_ptr<ErrorModelDiacriticsSimple> GlobalVars::errorModelDiacriticsSimpleP;
	boost::shared_ptr<ErrorModelDiacriticsComplex> GlobalVars::errorModelDiacriticsComplexP;
	boost::shared_ptr<ErrorModel> GlobalVars::errorModelTypoP;
	boost::shared_ptr<ErrorModelIgnoreDiacriticsTypo> GlobalVars::errorModelIgnoreDiacriticsTypoP;
	boost::shared_ptr<ErrorModelIgnoreDiacriticsTypo> GlobalVars::errorModelIgnoreDiacriticsTypoDiaCapZeroDistP;
	boost::shared_ptr<ErrorModelTypo_Dia_and_Cap_Zero_Dist> GlobalVars::errorModelTypoDiaCapZeroDistP;
	boost::shared_ptr<ErrorModelAbbreviation> GlobalVars::errorModelAbbreviationP;
	boost::shared_ptr<MorphologyMapper> GlobalVars::morphologyMapper;
	boost::shared_ptr<Lexicon> GlobalVars::lexicon;
	boost::shared_ptr<LMServer> GlobalVars::lmServer;
	boost::shared_ptr<StringMapper> GlobalVars::stringMapper;
	boost::shared_ptr<TextCheckingServer> GlobalVars::textCheckingServer;
	boost::shared_ptr<LemmaSigns> GlobalVars::lemmaSigns;
	boost::shared_ptr<MyCash_StagePosibility> GlobalVars::st_pos_multifactor_cash;
	boost::shared_ptr<MyCash_StagePosibility> GlobalVars::st_pos_singlefactor_cash;
	boost::shared_ptr<MyCash_StagePosibility> GlobalVars::st_pos_letter_cash;

		
}
