/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef GLOBALVARS_HPP_
#define GLOBALVARS_HPP_

#include "StdAfx.h"
#include "MyCash.hpp"

namespace ngramchecker {

	class ErrorModel;
	class ErrorModelDiacriticsSimple;
	class ErrorModelDiacriticsComplex;
	class ErrorModelTypo;
	class ErrorModelIgnoreDiacriticsTypo;
	class ErrorModelTypo_Dia_and_Cap_Zero_Dist;
	class ErrorModelAbbreviation;
	class MorphologyMapper;
	class Lexicon;
	class LMServer;
	class StringMapper;
	class TextCheckingServer;
	class LemmaSigns;
	class SimWordsFindex;
	class SimWordsFinderDiacritic;
	class SimWordsFinderSpelling;
	class StagePosibility;

	typedef MyCash<pair<string, bool>, vector<boost::shared_ptr<StagePosibility> > > MyCash_StagePosibility;


	class GlobalVars
	{
	public:

		static boost::shared_ptr<ErrorModelDiacriticsSimple> errorModelDiacriticsSimpleP;
		static boost::shared_ptr<ErrorModelDiacriticsComplex> errorModelDiacriticsComplexP;
		static boost::shared_ptr<ErrorModel> errorModelTypoP;
		static boost::shared_ptr<ErrorModelTypo_Dia_and_Cap_Zero_Dist> errorModelTypoDiaCapZeroDistP;
		static boost::shared_ptr<ErrorModelIgnoreDiacriticsTypo> errorModelIgnoreDiacriticsTypoP;
		static boost::shared_ptr<ErrorModelIgnoreDiacriticsTypo> errorModelIgnoreDiacriticsTypoDiaCapZeroDistP;
		static boost::shared_ptr<ErrorModelAbbreviation> errorModelAbbreviationP;
		static boost::shared_ptr<MorphologyMapper> morphologyMapper;
		static boost::shared_ptr<Lexicon> lexicon;
		static boost::shared_ptr<LMServer> lmServer;
		static boost::shared_ptr<StringMapper> stringMapper;
		static boost::shared_ptr<TextCheckingServer> textCheckingServer;
		static boost::shared_ptr<LemmaSigns> lemmaSigns;
		static boost::shared_ptr<MyCash_StagePosibility> st_pos_multifactor_cash;
		static boost::shared_ptr<MyCash_StagePosibility> st_pos_singlefactor_cash;
		static boost::shared_ptr<MyCash_StagePosibility> st_pos_letter_cash;

		static void InitGlobalVars();

	};

}

#endif /* GLOBALVARS_HPP_ */
