/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "LMServer.hpp"
#include "NGram.hpp"
#include "ZipLM.hpp"
#include "LMWrapper.hpp"
#include "LMWrapper_ZipLM.hpp"

namespace ngramchecker {

		LMServer::LMServer()
		{

			ZipLMP zlm1 = ZipLMP(new ZipLM(MyConstants::filename_form_zip_lm_bin));
			ZipLMP zlm2 = ZipLMP(new ZipLM(MyConstants::filename_lemma_zip_lm_bin));
			ZipLMP zlm3 = ZipLMP(new ZipLM(MyConstants::filename_tag_zip_lm_bin));

			//PackedLMP plm1 = PackedLMP(new PackedLM(filename_form_lm_bin));
			//PackedLMP plm2 = PackedLMP(new PackedLM(filename_lemma_lm_bin));
			//PackedLMP plm3 = PackedLMP(new PackedLM(filename_tag_lm_bin));


			ZipLMP zlm_letter_common = ZipLMP(new ZipLM(MyConstants::filename_letter_universal_zip_lm_bin));
			ZipLMP zlm_letter_names = zlm_letter_common;//ZipLMP(new ZipLM(filename_letter_universal_zip_lm_bin));
			//PackedLMP plm_letter = PackedLMP(new PackedLM(filename_letter_lm_bin));

			//lmWrappers.push_back(LMWrapperP(new LMWrapper_PackedLM(plm1)));
			//lmWrappers.push_back(LMWrapperP(new LMWrapper_PackedLM(plm2)));
			//lmWrappers.push_back(LMWrapperP(new LMWrapper_PackedLM(plm3)));

			lmWrappers.push_back(LMWrapperP(new LMWrapper_ZipLM(zlm1, MyConstants::default_ngram_cash_capacity, MyConstants::default_ngram_probs_cash_capacity)));
			lmWrappers.push_back(LMWrapperP(new LMWrapper_ZipLM(zlm2, MyConstants::default_ngram_cash_capacity, MyConstants::default_ngram_probs_cash_capacity)));
			lmWrappers.push_back(LMWrapperP(new LMWrapper_ZipLM(zlm3, MyConstants::default_ngram_cash_capacity, MyConstants::default_ngram_probs_cash_capacity)));

			letter_common_lmWrapper = LMWrapperP(new LMWrapper_ZipLM(zlm_letter_common, MyConstants::default_letter_ngram_cash_capacity, MyConstants::default_letter_ngram_probs_cash_capacity));
			letter_names_lmWrapper = LMWrapperP(new LMWrapper_ZipLM(zlm_letter_names, MyConstants::default_letter_ngram_cash_capacity, MyConstants::default_letter_ngram_probs_cash_capacity));
			//letter_lmWrapper = LMWrapperP(new LMWrapper_PackedLM(plm_letter));

		}

		double LMServer::GetNextWordCost(uint32_t factor_number, NGramP ngram_key)
		{
			return lmWrappers[factor_number]->GetProb(ngram_key);
		}

		double LMServer::GetContextFreeCost(uint32_t word_ID)
		{
			uint32_t* ngram_key_ids = new uint32_t[1];
			ngram_key_ids[0] = word_ID;

			NGramP ngram_key = NGram::createNGramSearchKey(1, ngram_key_ids);

			return lmWrappers[0]->GetProb(ngram_key);

		}

		double LMServer::GetNextLetterCost_names(NGramP letter_ngram_key)
		{
			return letter_names_lmWrapper->GetProb(letter_ngram_key);
		}

		double LMServer::GetNextLetterCost_common(NGramP letter_ngram_key)
		{
			return letter_common_lmWrapper->GetProb(letter_ngram_key);
		}

		bool LMServer::ExistFormBigram(uint32_t next_form_id, uint32_t prev_form_id)
		{
			uint32_t *bigram_ids = new uint32_t[2];
			bigram_ids[0] = next_form_id;
			bigram_ids[1] = prev_form_id;
			NGramP ngram_key = NGram::createNGramSearchKey(2, bigram_ids);
			return lmWrappers[0]->ContainsNGram(ngram_key);
		}


		double LMServer::GetMaxUnigramID(uint32_t factor)
		{
			return lmWrappers[factor]->MaxUnigramID();
		}

		void LMServer::ClearCash()
		{
			for (uint32_t i = 0; i < lmWrappers.size(); i++)
			{
					lmWrappers[i]->ClearCash();
			}
		}

		void LMServer::PrintStats()
		{
			cerr << "LMServer Stats:" << endl;

			for (uint32_t i = 0; i < 3; i++)
			{
					lmWrappers[i]->PrintStats();
			}
		}

	//LMServer lmServer;


}
