/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "LMWrapper_ZipLM.hpp"
#include "ZipLM.hpp"
#include "NGram.hpp"

namespace ngramchecker {

		NGramP LMWrapper_ZipLM::GetNGram(NGramP ngram_key)
		{
			NGramP ret;

			if (ngram_cash.IsCashed(ngram_key))
			{
				return ngram_cash.GetCashedValue(ngram_key);
			}

			NGramP lm_ngram = LM->GetNGramForNGramKey(ngram_key);

			uint32_t start_undef = (lm_ngram == NULL) ? 1 : lm_ngram->order + 1;
			uint32_t end_undef = ngram_key->order;

			for (uint32_t undef_order = start_undef; undef_order <= end_undef; undef_order++)
			{
				uint32_t* pom_ids = new uint32_t[undef_order];

				for (uint32_t j = 0; j < undef_order; j++)
					pom_ids[j] = ngram_key->GetWordID_New(j);

				NGramP undef_key = NGram::createNGramSearchKey(undef_order, pom_ids);

				if (! ngram_cash.IsCashed(undef_key))
					ngram_cash.StoreValueForKey(undef_key, NGramP());
			}

			if (lm_ngram != NULL)
			{
				if (lm_ngram->order == ngram_key->order)
				{
					ngram_cash.StoreValueForKey(ngram_key, lm_ngram);
					ret = lm_ngram;
				}
				else
				{

					if (! ngram_cash.IsCashed(lm_ngram) )
						ngram_cash.StoreValueForKey(lm_ngram, lm_ngram);
				}


			}
			return ret;
		}

		LMWrapper_ZipLM::LMWrapper_ZipLM(ZipLMP _LM, uint32_t ngram_cash_capacity, uint32_t ngram_probs_cash_capacity):
			LM(_LM),
			_lookups(0),
			_hits(0),
			ngram_cash(ngram_cash_capacity),
			ngram_probs_cash(ngram_probs_cash_capacity)
		{}

		double LMWrapper_ZipLM::GetProb_old(NGramP ngram_key)
		{
			double ret = 0;

			//cerr << "Zip LM Wrapper In!" << endl;

			NGramP ngram_key_backup = ngram_key;
			NGramP ngram = GetNGram(ngram_key);


			while (ngram == NULL)
			{
				//cerr << ngram_key->order << "gram not found, backing off.." << endl;
				if (ngram_key->order == 1)
				{
					ret += LM->GetWordNotInLMCost();
					return ret;
				}

				uint32_t *history_ids = new uint32_t[ngram_key->order - 1];
				for (uint32_t i = 1; i < ngram_key->order; i++)
					history_ids[i - 1] = ngram_key->GetWordID_New(i);

				NGramP bow_ngram_key = NGram::createNGramSearchKey(ngram_key->order - 1, history_ids);

				NGramP bow_ngram = GetNGram(bow_ngram_key);

				if (bow_ngram != NULL)
				{
					ret += bow_ngram->backoff;
					//cerr << "BOW ngram found, new ret value = " << ret << endl;
				}
				else
				{
					//cerr << "BOW not found" << endl;
				}

				uint32_t *lower_order_ids = new uint32_t[ngram_key->order - 1];
				for (uint32_t i = 0; i < ngram_key->order - 1; i++)
					lower_order_ids[i] = ngram_key->GetWordID_New(i);

				NGramP lower_order_key = NGram::createNGramSearchKey(ngram_key->order - 1, lower_order_ids);
				ngram_key = lower_order_key;
				//ngram_key->order = ngram_key->order - 1;

				ngram = GetNGram(ngram_key);
			}

			ret += ngram->prob;
			//cerr << ngram->order << "gram founded, ngram->prob = " << ngram->prob <<
			//		", final prob value = " << ret << endl << "GetProb finished" << endl << endl;

			//cerr << "Prob of: " << endl << ngram_key_backup->ToString() << "is: " << ret << endl;

			//cerr << "out!" << endl;

			return ret;
		}


		double LMWrapper_ZipLM::GetProb(NGramP ngram_key)
		{
			double ret = 0;

			if (ngram_probs_cash.IsCashed(ngram_key))
			{
				ret += ngram_probs_cash.GetCashedValue(ngram_key);

				//double old_prob = GetProb_old(ngram_key);
				//assert(abs(old_prob - ret) < 0.001);

				return ret;
			}


			if (ngram_cash.IsCashed(ngram_key))
			{
				NGramP ngram_val = ngram_cash.GetCashedValue(ngram_key);

				if (ngram_val != NULL)
				{
					ret += ngram_val->prob;
					//ngram_probs_cash.StoreValueForKey(ngram_key, ret);

					//double old_prob = GetProb_old(ngram_key);
					//assert(abs(old_prob - ret) < 0.001);

					return ret;
				}
				//assert(ngram_cash.GetCashedValue(ngram_key) == NULL);
			}
			else
			{
				NGramP ngram_val = GetNGram(ngram_key);

				if (ngram_val != NULL)
				{

					ret += ngram_val->prob;
					//ngram_probs_cash.StoreValueForKey(ngram_key, ret);

					//double old_prob = GetProb_old(ngram_key);
					//assert(abs(old_prob - ret) < 0.001);

					return ret;
				}
			}

			if (ngram_key->order == 1)
			{
				ret += LM->GetWordNotInLMCost();
				//ngram_probs_cash.StoreValueForKey(ngram_key, ret);

				//double old_prob = GetProb_old(ngram_key);
				//assert(abs(old_prob - ret) < 0.001);

				return ret;
			}

			uint32_t *history_ids = new uint32_t[ngram_key->order - 1];
			for (uint32_t i = 1; i < ngram_key->order; i++)
				history_ids[i - 1] = ngram_key->GetWordID_New(i);

			NGramP bow_ngram_key = NGram::createNGramSearchKey(ngram_key->order - 1, history_ids);

			NGramP bow_ngram = GetNGram(bow_ngram_key);

			if (bow_ngram != NULL)
			{
				ret += bow_ngram->backoff;

				//if (! ngram_probs_cash.IsCashed(bow_ngram_key))
				//	ngram_probs_cash.StoreValueForKey(bow_ngram_key, bow_ngram->prob);
			}

			uint32_t *lower_order_ids = new uint32_t[ngram_key->order - 1];
			for (uint32_t i = 0; i < ngram_key->order - 1; i++)
				lower_order_ids[i] = ngram_key->GetWordID_New(i);

			NGramP lower_order_key = NGram::createNGramSearchKey(ngram_key->order - 1, lower_order_ids);

			ret += GetProb(lower_order_key);

			ngram_probs_cash.StoreValueForKey(ngram_key, ret);

			//double old_prob = GetProb_old(ngram_key);
			//assert(abs(old_prob - ret) < 0.001);

			return ret;
		}


		bool LMWrapper_ZipLM::ContainsNGram(NGramP ngram_key)
		{
			NGramP ngram = GetNGram(ngram_key);

			return (ngram != NULL);
		}

		void LMWrapper_ZipLM::ClearCash()
		{
			//ngram_cash.clear();
			//cashed_ids.clear();
		}

		void LMWrapper_ZipLM::PrintStats()
		{
			cerr << "LMWrapper_ZipLM \"" << LM->GetFilename() << "\" Stats:" << endl << "lookups = " << _lookups << endl << "hits = " << _hits << endl;
		}

		uint32_t LMWrapper_ZipLM::MaxUnigramID()
		{
			return LM->MaxUnigramID();
		}

}