/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "LemmaSigns.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "MorphologyMapper.hpp"
#include "MyPackedArray.hpp"

namespace ngramchecker {

		uint32_t LemmaSigns::get_signature_for_lemma_sign_string(const string &str)
		{
			uint32_t mult = 1;
			uint32_t pom_sig = 0;
			for (uint32_t j = 0; j < str.length(); j++)
			{
				if (str[j] != '-')
					pom_sig += mult;
				mult *= 2;
			}

			return pom_sig;

		}

		uint32_t LemmaSigns::get_signature_for_lemma_sign_symbol_list(const string &str)
		{
			uint32_t sig = 0;

			for (uint32_t i = 0; i < str.length(); i++)
			{
				uint32_t index = MyConstants::global_lemma_signs_symbols.find(str[i], 0);

				if (index != string::npos)
				{
					sig += (1 << index);
				}
			}

			return sig;
		}

		string LemmaSigns::get_string_for_lemma_sign_signature(const uint32_t sig)
		{
			uint32_t mult = 1;
			string ret = "";

			for (uint32_t i = 0; i < MyConstants::global_lemma_signs_symbols.length(); i++)
			{
				if ( (mult & sig) != 0)
					ret += MyConstants::global_lemma_signs_symbols[i];
				else
					ret += '-';

				mult *= 2;
			}

			return ret;
		}

		void LemmaSigns::createBinaryFiles()
		{
			string s;
			ofstream ofs;
			ofs.open( MyConstants::filename_lemma_signs.c_str(), ios::binary | ios::out );
			assert(ofs.is_open());

			ifstream sg_input;
			sg_input.open( (MyConstants::project_home + "/data/morphology/text/signs_groups.txt").c_str() );
			assert(sg_input.is_open());

			vector<string> sg_vec;

			while (std::getline(sg_input, s))
				sg_vec.push_back(s);

			uint32_t num_sg_groups = sg_vec.size();

			ofs.write((char*)&num_sg_groups, sizeof(uint32_t));

			for (uint32_t i = 0; i < num_sg_groups; i++)
			{
				uint32_t pom_sig = get_signature_for_lemma_sign_string(sg_vec[i]);
				ofs.write((char*)&pom_sig, sizeof(uint32_t));
			}

			sg_input.close();

			ifstream ls_input;
			ls_input.open( (MyConstants::project_home + "/data/morphology/text/lemma_signs.txt").c_str() );
			assert(ls_input.is_open());

			vector<uint32_t> lemmas_signs;


			for (uint32_t i = 0; i < GlobalVars::stringMapper->NumLemmas(); i++)
				lemmas_signs.push_back(0);



			//uint32_t pom_counter = 0;
			while (std::getline(ls_input, s))
			{
				vector<string> parts;

				boost::split(parts, s, boost::is_any_of(" "));

				if (GlobalVars::stringMapper->ContainsLemma(parts[0]))
				{
					lemmas_signs[GlobalVars::stringMapper->MapLemma(parts[0])] = boost::lexical_cast<uint32_t>(parts[1]);
				}
				else
					cerr << "unknown lemma: " << parts[0] << endl;
				//pom_counter++;

				//lemmas_signs.push_back(boost::lexical_cast<uint32_t>(parts[1]));
			}


			boost::shared_ptr<MyPackedArray> lemma_signs_pa = boost::shared_ptr<MyPackedArray>(new MyPackedArray(lemmas_signs));
			lemma_signs_pa->WriteToStream(ofs);


			ofs.close();
			cerr << "OK!";
		}

		LemmaSigns::LemmaSigns():
			lemma_signs_cash(MyConstants::default_lemma_signs_cash_capacity)
		{
			ifstream ifs;
			ifs.open( MyConstants::filename_lemma_signs.c_str(), ios::binary | ios::out );
			assert(ifs.is_open());

			uint32_t num_lemma_signs_groups;
			ifs.read((char*)&num_lemma_signs_groups, sizeof(uint32_t));

			for (uint32_t i = 0; i < num_lemma_signs_groups; i++)
			{
				uint32_t val;
				ifs.read((char*)&val, sizeof(uint32_t));
				groups.push_back(val);
			}

			lemma_signs = boost::shared_ptr<MyPackedArray>(new MyPackedArray(ifs));

			current_conditions_signature = 511;

			string desc[] = {"zastaralé", "expresivní", "hovorové", "slangové",
					   "nářečí", "knižní", "cizí slovo", "vulgární", "zastaralý pravopis" };

			for (uint32_t i = 0; i < MyConstants::global_lemma_signs_symbols.length(); i++)
			{
				describtions.push_back(desc[i]);
			}
		}

		vector<string> LemmaSigns::GetSignDescribtions()
		{

			vector<string> ret;

			for (uint32_t i = 0; i < MyConstants::global_lemma_signs_symbols.length(); i++)
			{
				stringstream strs;
				strs << MyConstants::global_lemma_signs_symbols[i] << " - " << describtions[i];
				ret.push_back(strs.str());
			}

			return ret;
		}

		vector<string> LemmaSigns::GetCurrentStateDescribtion()
		{
			uint32_t pom = 1;

			stringstream allowed;
			stringstream disallowed;

			allowed << "Povolené příznaky : ";
			disallowed << "Zakázané příznaky : ";


			vector<bool> sa_vec;

			for (uint32_t i = 0; i < MyConstants::global_lemma_signs_symbols.length(); i++)
			{
				if ( (pom & current_conditions_signature) != 0)
				{
					sa_vec.push_back(true);
					allowed << MyConstants::global_lemma_signs_symbols[i];
				}
				else
				{
					sa_vec.push_back(false);
					disallowed << MyConstants::global_lemma_signs_symbols[i];
				}
				pom = pom << 1;
			}

			allowed << " (";
			disallowed << " (";

			bool al_first = true;
			bool dis_first = true;

			for (uint32_t i = 0; i < sa_vec.size(); i++)
			{
				if (sa_vec[i] == true)
				{
					if (!al_first)
						allowed << ", ";

					al_first = false;
					allowed << describtions[i];
				}
				else
				{
					if (!dis_first)
						disallowed << ", ";

					dis_first = false;
					disallowed << describtions[i];
				}
			}

			allowed << ")";
			disallowed << ")";

			vector<string> ret;
			ret.push_back(allowed.str());
			ret.push_back(disallowed.str());

			return ret;
		}

		bool LemmaSigns::IsLemmaOkey(uint32_t lemmaID)
		{
			if (lemmaID > lemma_signs->GetSize())
				return true;

			uint32_t lemma_signature = groups[lemma_signs->GetValueAt(lemmaID)];

			return  (lemma_signature | current_conditions_signature) == current_conditions_signature;
		}

		bool LemmaSigns::IsFormOkey(uint32_t formID)
		{
			if (lemma_signs_cash.IsCashed(formID))
			{
				return lemma_signs_cash.GetCashedValue(formID);
			}

			vectorP_WordFactors wfs = GlobalVars::morphologyMapper->GetOutputArcsPerWord(formID, false);

			for (uint32_t i = 0; i < wfs->size(); i++)
			{
				if (IsLemmaOkey(wfs->operator [](i).factors[1]))
				{
					lemma_signs_cash.StoreValueForKey(formID, true);
					return true;
				}
			}

			lemma_signs_cash.StoreValueForKey(formID, false);
			return false;
		}

		void LemmaSigns::SetConditionsSignature(uint32_t new_conditions_signature)
		{
			current_conditions_signature = new_conditions_signature;
		}

		void LemmaSigns::SetConditionsSignature(const string &symbols_list)
		{
			current_conditions_signature = get_signature_for_lemma_sign_symbol_list(symbols_list);
		}

		void LemmaSigns::new_functions_test()
		{
			cerr << "sig = 60 => " << get_string_for_lemma_sign_signature(60) << endl;
		}

		void LemmaSigns::AllowSigns(string signs)
		{
			current_conditions_signature = (current_conditions_signature | get_signature_for_lemma_sign_symbol_list(signs));
		}

		void LemmaSigns::DisallowSigns(string signs)
		{
			current_conditions_signature -= (current_conditions_signature & get_signature_for_lemma_sign_symbol_list(signs));
		}
}