/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef LEMMASIGNS_HPP_
#define LEMMASIGNS_HPP_

#include "StdAfx.h"
#include "MyCash.hpp"

namespace ngramchecker {

	class MyPackedArray;
	SP_DEF(MyPackedArray);

	class LemmaSigns {

	private:

		vector<uint32_t> groups;
		boost::shared_ptr<MyPackedArray> lemma_signs;
		uint32_t current_conditions_signature;
		vector<string> describtions;
		MyCash<uint32_t, bool> lemma_signs_cash;

		static uint32_t get_signature_for_lemma_sign_string(const string &str);

		static uint32_t get_signature_for_lemma_sign_symbol_list(const string &str);

		static string get_string_for_lemma_sign_signature(const uint32_t sig);

	public:


		static void createBinaryFiles();

		LemmaSigns();

		vector<string> GetSignDescribtions();

		vector<string> GetCurrentStateDescribtion();

		bool IsLemmaOkey(uint32_t lemmaID);

		bool IsFormOkey(uint32_t formID);

		void SetConditionsSignature(uint32_t new_conditions_signature);

		void SetConditionsSignature(const string &symbols_list);

		static void new_functions_test();

		void AllowSigns(string signs);

		void DisallowSigns(string signs);
	};

}

#endif /* LEMMASIGNS_HPP_ */
