/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef _LEXICON_HPP_
#define _LEXICON_HPP_

#include "StdAfx.h"
#include "MyCash.hpp"
#include "LexiconArc.hpp"

namespace ngramchecker {

	class MyPackedArray;
	SP_DEF(MyPackedArray);

	class CompIncreasingArray;
	SP_DEF(CompIncreasingArray);

	class ErrorModel;
	SP_DEF(ErrorModel);

	typedef MyCash<uint32_t, vectorP_LexiconArc> MyCash_LexiconStates;

	class Lexicon {

	private:

	  CompIncreasingArrayP  states_arcpointer;
	  MyPackedArrayP arcs_nextstate;
	  MyPackedArrayP arcs_isymbol;

	  uint32_t n_inner_states;

	  MyCash_LexiconStates lexicon_cash;

	  Lexicon(uint32_t _n_inner_states, boost::shared_ptr<CompIncreasingArray> _states_arcpointer, boost::shared_ptr<MyPackedArray> _arcs_nextstate,
			  boost::shared_ptr<MyPackedArray> _arcs_isymbol): states_arcpointer(_states_arcpointer), arcs_nextstate(_arcs_nextstate), arcs_isymbol(_arcs_isymbol), n_inner_states(_n_inner_states),
			  lexicon_cash(MyConstants::default_lexicon_state_cash_capacity)
	  {}

	  uint32_t NumStates() const;

	  boost::shared_ptr<set<LexiconArc> > Arcs_set(uint32_t stateID);

	  vectorP_LexiconArc Arcs_vector(uint32_t stateID);

	  uint32_t GetWordID_per_state_id(uint32_t state_ID);

	  int ContainsSuffix(uint32_t stateID, const string &word, uint32_t start_char_index, bool check_lemma_sign);

	  bool StateIsWord(uint32_t state_ID, bool check_lemma_sign);

	  int SingleArc_nextstate(uint32_t stateID, char character);

	public:

	  static Lexicon* ReadFromBinFile(string filename);

	  void WriteToBinFile(string filename);

	  int GetWordID(const string &word_iso); //return value -1 for out of vocabulary words!

	  Lexicon(map<uint32_t, map<uint32_t, uint32_t> > states_map, uint32_t _n_inner_states);

	  void TestIt();

	private:

	  void AddSimilarWordToMap(Similar_Words_Map &ret, uint32_t word_id, double cost, string &word, uint32_t word_include_letter_start_index, string &prefix);

	  Similar_Words_Map GetSimilarWords_impl(string &word, uint32_t edit_distance, double cost, ErrorModelP errModel, uint32_t stateID, uint32_t startIndex, string &prefix, double cost_limit, bool ignore_case, uint32_t max_edit_distance);

	public:

	  Similar_Words_Map GetSimilarWords(string word, uint32_t max_ed_dist, double _cost_limit, ErrorModelP errModel, bool _ignore_case);
	  vector<string> GetSimilarWords_string(string word, uint32_t max_ed_dist, double _cost_limit, ErrorModelP errModel, bool _ignore_case);
	};


	SP_DEF(Lexicon);

}

#endif  //_LEXICON_HPP_
