/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef MORPHOLOGY_MAPPER_HPP_
#define MORPHOLOGY_MAPPER_HPP_

#include "StdAfx.h"
#include "MyCash.hpp"

namespace ngramchecker {

	class MyPackedArray;
	SP_DEF(MyPackedArray);

	class CompIncreasingArray;
	SP_DEF(CompIncreasingArray);

	class MyMappedDoubleArray;
	SP_DEF(MyMappedDoubleArray);

	struct WordFactors {
	public:
		uint32_t factors[3];
		double emmision_probs[3];

		WordFactors(uint32_t _formID, uint32_t _lemmaID, uint32_t _tagID, double lemma_prob, double tag_prob);

		string ToString();

	};

	typedef boost::shared_ptr<vector<WordFactors> > vectorP_WordFactors;

	typedef pair<WordFactors, double> WF_Cost_Pair;
	typedef vector<WF_Cost_Pair> vector_WF_Cost_Pair;
	typedef boost::shared_ptr<vector<WF_Cost_Pair> > vectorP_WF_Cost_Pair;

	typedef MyCash<pair<uint32_t, bool>, vectorP_WordFactors> MyCash_Morphology;

	class MorphologyMapper {
	private:

		vector< vector<uint32_t>  > MA_tagGroupsDefs;

		CompIncreasingArrayP MA_offsets;
		MyPackedArrayP MA_lemmas;
		MyPackedArrayP MA_tagGroups;

		MyMappedDoubleArrayP EM_lemma;
		MyMappedDoubleArrayP EM_tag;

		CompIncreasingArrayP EM_tag_offsets;

		uint32_t universal_sentence_start;
		uint32_t universal_sentence_end;

		uint32_t universal_number;
		uint32_t universal_name;
		uint32_t universal_unk;
		uint32_t universal_first_non_special;
		uint32_t universal_punctuation_start;

		MyCash_Morphology morphology_cash;

	public:

		uint32_t NumForms();

		uint32_t NumLemmas();

		uint32_t GetSentenceStartID();

		uint32_t GetSentenceEndID();

		uint32_t GetNumberTokenID();

		uint32_t GetNameTokenID();

		uint32_t GetUnkTokenID();

		uint32_t GetFirstNonSpecialID();

		static void createBinaryFiles();

		MorphologyMapper(bool init_emmisions, uint32_t cash_size);

		vectorP_WordFactors GetOutputArcsPerWord(uint32_t formID, bool use_tag);

		vectorP_WordFactors GetOutputArcsPerWordNoEmmisions(uint32_t formID, bool use_tag);

		void MakeEmmisionSmoothingTextFile();

		void MakeWordEmissionBinFilesFromTextFiles(uint32_t bits_per_lemma, uint32_t bits_per_tag);

		uint32_t PunctuationUniversalIndex_char(char character);

		bool IsNormalWord(uint32_t form_id);

		void TestIt(string outfilename);
	};

	SP_DEF(MorphologyMapper);

	//MorphologyMapper morphologyMapper;

}

#endif /* MORPHOLOGY_MAPPER_HPP_ */
