/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "MyIncreasingArray.hpp"
#include "MyPackedArray.hpp"

namespace ngramchecker {

		uint32_t MyIncreasingArray::GetSize()
		{
			return shifted_offsets->GetSize();
		}

		uint32_t MyIncreasingArray::GetValueAt(uint32_t i)
		{
			return i * multiplier / denominator + shifted_offsets->GetValueAt(i) + value_shift;
		}

		MyIncreasingArray::MyIncreasingArray(ifstream &ifs)
		{
			assert(ifs.is_open());

			ifs.read((char*)&multiplier, sizeof(uint32_t));
			ifs.read((char*)&denominator, sizeof(uint32_t));
			ifs.read((char*)&value_shift, sizeof(uint32_t));
			shifted_offsets = boost::shared_ptr<MyPackedArray>(new MyPackedArray(ifs));

		}

		void MyIncreasingArray::WriteToStream(ofstream &ofs)
		{
			assert(ofs.is_open());

			ofs.write((char*)&multiplier, sizeof(uint32_t));
			ofs.write((char*)&denominator, sizeof(uint32_t));
			ofs.write((char*)&value_shift, sizeof(uint32_t));
			shifted_offsets->WriteToStream(ofs);
		}

		MyIncreasingArray::MyIncreasingArray(vector<uint32_t> &vec)
		{
			uint32_t max_val = vec[vec.size() - 1];
			uint32_t min_val = vec[0];

			for (uint32_t i = 0; i < vec.size() - 1; i++)
			{
				assert(vec[i] <= vec[i + 1]);
			}

			double ratio = (double)(max_val - min_val) / (double)(vec.size() - 1);


			double minDif = 1000000;
			uint32_t actDenom;
			uint32_t bestDenom;
			uint32_t bestMult;
			for (uint32_t i = 0; i < 10000; i++)
			{
				actDenom = (uint32_t)(i / ratio);
				double dif = fabs((double)i / actDenom - ratio);
				if (dif < minDif)
				{
					minDif = dif;
					bestDenom = actDenom;
					bestMult = i;
				}
			}

			//cerr << "initialization of MyIncreasingArray: multiplier = " << bestMult << ", denominator = " << bestDenom << endl;
			//cerr << "ratio = " << ratio << ", difference = " << minDif << endl;

			multiplier = bestMult;
			denominator = bestDenom;

			vector<int> voffsets;

			int min_offset = 10000000;

			for (uint32_t i = 0; i < vec.size(); i++)
			{
				int offs = vec[i] - (i * multiplier / denominator + min_val);
				voffsets.push_back(offs);

				if (offs < min_offset)
					min_offset = offs;

			}

			vector<uint32_t> sh_offsets;

			for (uint32_t i = 0; i < voffsets.size(); i++)
			{
				assert(voffsets[i] - min_offset >= 0);

				uint32_t sh_offset = voffsets[i] - min_offset;
				sh_offsets.push_back(sh_offset);
			}

			shifted_offsets = boost::shared_ptr<MyPackedArray>(new MyPackedArray(sh_offsets));
			value_shift = min_val + min_offset;

			//cerr << "MyIncreasingArray created: bitsPerValue = " << shifted_offsets->GetBitsPerValue() << endl;
			//cerr << "MyPackedArray would need " << BitsNeeded(max_val - min_val) << " bits\n----\n";

			for (uint32_t i = 0; i < vec.size(); i++)
			{
				assert(vec[i] == GetValueAt(i));
			}
		}

}