/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "MyMappedDoubleArray.hpp"
#include "ValueMapping.hpp"
#include "MyPackedArray.hpp"

namespace ngramchecker {

		uint32_t MyMappedDoubleArray::GetSize()
		{
			return mpa->GetSize();
		}

		double MyMappedDoubleArray::GetValueAt(uint32_t index)
		{
			return value_mapping->GetDouble(mpa->GetValueAt(index));
		}

		void MyMappedDoubleArray::WriteToStream(ofstream &ofs)
		{
			assert(ofs.is_open());
			value_mapping->writeToStream(ofs);
			mpa->WriteToStream(ofs);
		}

		MyMappedDoubleArray::MyMappedDoubleArray(vector<double> &values, uint32_t bits_per_value)
		{
			value_mapping = ValueMappingP(new ValueMapping(values, bits_per_value));

			vector<uint32_t> int_vals;

			for (uint32_t i = 0; i < values.size(); i++)
			{
				int_vals.push_back(value_mapping->GetCenterID(values[i]));
			}

			mpa = MyPackedArrayP(new MyPackedArray(int_vals));
		}

		MyMappedDoubleArray::MyMappedDoubleArray(ifstream &ifs)
		{
			assert(ifs.is_open());

			value_mapping = ValueMappingP(ValueMapping::readFromStream(ifs));
			mpa = MyPackedArrayP(new MyPackedArray(ifs));
		}

}
