/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "MyPackedArray.hpp"

namespace ngramchecker {

			void MyPackedArray::PrintBits()
			{
				for (uint32_t i = 0; i < num_bytes; i++)
				{
					for (uint32_t j = 0; j < 8; j++)
					{
						cout << (1 & (data[i] >> j));
					}
					cout << endl;
				}
			}


			uint32_t MyPackedArray::GetSize()
			{
				return num_values;
			}

			uint32_t MyPackedArray::GetNumberOfBytes()
			{
				return num_bytes;
			}

			uint32_t MyPackedArray::GetBitsPerValue()
			{
				return bits_per_value;
			}

			uint32_t MyPackedArray::GetValueAt(uint32_t index)
			{
				ulong64 retVal = 0;
				unsigned char* ukaz = (unsigned char*)&retVal;

				uint32_t byte_pointer = (index * bits_per_value) >> 3;
				uint32_t bit_pointer = (index * bits_per_value) % 8;

				int bits_left = bits_per_value;
				ukaz[0] = data[byte_pointer];
				bits_left -= 8 - bit_pointer;
				uint32_t offset = 1;

				while (bits_left > 0)
				{
					bits_left -= 8;
					ukaz[offset] = data[byte_pointer + offset];
					offset++;
				}

				retVal = (retVal >> bit_pointer) & output_mask;

				return retVal;
			}

			MyPackedArray::~MyPackedArray()
			{
				delete[] data;
			}

			MyPackedArray::MyPackedArray(const MyPackedArray& val)
			{
				bits_per_value = val.bits_per_value;
				num_bytes = val.num_bytes;
				num_values = val.num_values;
				output_mask = val.output_mask;

				data = new unsigned char[num_bytes];

				for (uint32_t i = 0; i < num_bytes; i++)
				{
					data[i] = val.data[i];
				}

				cerr << "MyPackedArray: copy constructor\n";
			}

			MyPackedArray& MyPackedArray::operator=(const MyPackedArray& val)
			{
				if (this != &val)
				{
					unsigned char* new_data = new unsigned char[val.num_bytes];

					memcpy(new_data, val.data, val.num_bytes);

					bits_per_value = val.bits_per_value;
					num_bytes = val.num_bytes;
					num_values = val.num_values;
					output_mask = val.output_mask;

					delete[] data;
					data = new_data;

				}

				cerr << "MyPackedArray: assignemt operator\n";
				return *this;
			}

			MyPackedArray::MyPackedArray(ifstream &ifs)
			{
				assert(ifs.is_open());

				ifs.read((char*)&bits_per_value, sizeof(uint32_t));
				ifs.read((char*)&num_bytes, sizeof(uint32_t));
				ifs.read((char*)&num_values, sizeof(uint32_t));
				ifs.read((char*)&output_mask, sizeof(uint32_t));

				data = new unsigned char[num_bytes];

				ifs.read((char*)data, sizeof(unsigned char) * num_bytes);

			}

			MyPackedArray::MyPackedArray(const string filename)
			{
				ifstream ifs;
				ifs.open(filename.c_str(), ios::in | ios::binary);

				if (! ifs.is_open())
				{
					cerr << "Can't open file: " << filename;
					exit(1);
				}
				ifs.read((char*)&bits_per_value, sizeof(uint32_t));
				ifs.read((char*)&num_bytes, sizeof(uint32_t));
				ifs.read((char*)&num_values, sizeof(uint32_t));
				ifs.read((char*)&output_mask, sizeof(uint32_t));

				data = new unsigned char[num_bytes];

				ifs.read((char*)data, sizeof(unsigned char) * num_bytes);

				ifs.close();
			}

			void MyPackedArray::WriteToStream(ofstream &ofs)
			{
				assert(ofs.is_open());
				ofs.write((char*)&bits_per_value, sizeof(uint32_t));
				ofs.write((char*)&num_bytes, sizeof(uint32_t));
				ofs.write((char*)&num_values, sizeof(uint32_t));

				ofs.write((char*)&output_mask, sizeof(uint32_t));
				ofs.write((char*)data, sizeof(unsigned char) * num_bytes);

			}

			void MyPackedArray::SaveToFile(const string filename)
			{
				ofstream ofs;
				ofs.open(filename.c_str(), ios::out | ios::binary);

				if (! ofs.is_open())
				{
					cerr << "Can't open file: " << filename;
					exit(1);
				}

				cout << filename << " opened: number of bytes:" << num_bytes << endl;
				cout << "bits_per_value: " << bits_per_value << ", num_values: " << num_values << endl;
				cout << "output_mask: " << output_mask << endl;

				WriteToStream(ofs);

				ofs.close();
			}


			MyPackedArray::MyPackedArray(const vector<uint32_t> &values)
			{
				uint32_t max = 0;
				for (uint32_t i = 0; i < values.size(); i++)
				{
					if (values[i] > max) max = values[i];
				}

				uint32_t num_bits = 0;
				uint32_t pom = 1;
				while (max + 1 > pom)
				{
					pom = pom << 1;
					num_bits++;
				}

				num_bytes = (num_bits * values.size() + 7) / 8;

				bits_per_value = num_bits;
				num_values = values.size();

				output_mask = 0;
				for (uint32_t i = 0; i < num_bits; i++)
				{
					output_mask = output_mask << 1;
					output_mask++;
				}

				data = new unsigned char[num_bytes];

				for (uint32_t i = 0; i < num_bytes; i++)
					data[i] = 0;

				uint32_t byte_pointer;
				uint32_t bit_pointer;
				ulong64 curr_value;
				unsigned char* ukaz;
				for (uint32_t i = 0; i < values.size(); i++)
				{
					int bites_left = bits_per_value;
					byte_pointer = (i * num_bits) >> 3;
					bit_pointer = (i * num_bits) % 8;

					curr_value = values[i];

					curr_value = curr_value << bit_pointer;
					ukaz = (unsigned char*)&curr_value;
					bites_left -= 8 - bit_pointer;
					data[byte_pointer] = data[byte_pointer] | ukaz[0];

					uint32_t j = 1;
					while (bites_left > 0)
					{
						data[byte_pointer + j] = data[byte_pointer + j] | ukaz[j];
						bites_left -= 8;
						j++;
					}
				}

				for (uint32_t i = 0; i < values.size(); i++)
				{
					uint32_t val = GetValueAt(i);
					if (val != values[i])
					{
						for (uint j = 0; j < values.size(); j++)
							cerr << "values[" << j << "] = " << values[j] << endl;
						cerr << "val == " << val << ", values[" << i << "] == " << values[i] << endl;
					}
					assert(val == values[i]);
				}

				//PrintBits();

			}
}
