/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "NGram.hpp"

namespace ngramchecker {

		NGram::NGram(const NGram &val):
			order(val.order), prob(val.prob), backoff(val.backoff)
		{
			word_ids = new uint32_t[order];
			for (uint32_t i = 0; i < order; i++)
			{
				word_ids[i] = val.word_ids[i];
			}
		}

		NGram& NGram::operator=(const NGram& val)
		{
			if (&val != this)
			{
				order = val.order;
				backoff = val.backoff;
				prob = val.prob;

				delete[] word_ids;

				word_ids = new uint32_t[order];

				for (uint32_t i = 0; i < order; i++)
				{
					word_ids[i] = val.word_ids[i];
				}

			}

			return *this;
		}

		NGram::NGram(uint32_t order, uint32_t *IDs, double _prob, double _backoff):
			order(order), prob(_prob), backoff(_backoff), word_ids(IDs)
		{}

		bool NGram::ZeroBackoff() const
		{
			//assert(1 == 0); //To be implemented!
			return false;
		}

		NGram::~NGram()
		{
			delete[] word_ids;
		}

		NGramP NGram::createNGramPointer(uint32_t order, uint32_t* ids, double prob, double backoff)
		{
			return NGramP(new NGram(order, ids, prob, backoff));
		}

		NGramP NGram::createNGramSearchKey(uint32_t order, uint32_t *ids)
		{
			return createNGramPointer(order, ids, 0, 0);
		}

		string NGram::ToString()
		{
			stringstream strs;
			strs << "order: " << order << endl;
			strs << "next_word: " << word_ids[0] << endl;
			strs << "history (youngest -> oldest):";
			for (uint32_t i = 1; i < order; i++)
				strs << " " << word_ids[i];
			strs << endl << "prob = " << prob << ", bow = " << backoff << endl;
			return strs.str();
		}
}