/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef NGRAM_HPP_
#define NGRAM_HPP_

#include "StdAfx.h"

namespace ngramchecker {

	class NGram;

	SP_DEF(NGram);
	typedef boost::shared_ptr<vector<NGramP> > vectorP_NGramP;

	class NGram {

	public:
		uint32_t order;
		double prob;
		double backoff;

	private:
		uint32_t *word_ids;

		NGram(const NGram &val);

		NGram& operator=(const NGram& val);

		NGram(uint32_t order, uint32_t *IDs, double _prob, double _backoff);

	public:

		uint32_t GetWordID_New(uint32_t index) //index = 0 ~ next word, index = 1 ~ most recent history etc...
		{
			return word_ids[index];
		}

		bool ZeroBackoff() const;

		~NGram();

		static NGramP createNGramPointer(uint32_t order, uint32_t* ids, double prob, double backoff);

		static NGramP createNGramSearchKey(uint32_t order, uint32_t *ids);

		string ToString();
	};




	struct NGramP_iequal_to
		: std::binary_function<NGramP, NGramP, bool>
	{
		bool operator()(NGramP const& x,
			NGramP const& y) const
		{
			if (x->order != y->order)
				return false;


			for (uint32_t i = 0; i < x->order; i++)
			{
				if (x->GetWordID_New(i) != y->GetWordID_New(i))
					return false;
			}

			return true;
		}
	};


	struct NGramP_ihash: std::unary_function<NGramP, std::size_t>
	{
		std::size_t operator()(NGramP const& x) const
		{
			std::size_t seed = 0;

			boost::hash_combine(seed, x->order);

			for (uint32_t i = 0; i < x->order; i++)
			{
				boost::hash_combine(seed, x->GetWordID_New(i));
			}

			return seed;
		}
	};

}

#endif /* NGRAM_HPP_ */
