/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "SimWordsFinder.hpp"
#include "Token.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "Lexicon.hpp"
#include "MorphologyMapper.hpp"
#include "ErrorModel.hpp"

namespace ngramchecker {


		Similar_Words_Map SimWordsFinder::Find_basic(Token &token, bool sentence_start, uint32_t lookup_max_ed_dist, double lookup_max_cost)
		{
			string word_str = token.GetWordStringISO();
			Similar_Words_Map msw;

			if (token.ID == GlobalVars::morphologyMapper->GetNumberTokenID() ||
				(word_str.length() == 1 && GlobalVars::stringMapper->CharIsPunctuation(word_str[0]) ) )
			{

				stringP word_strP = stringP(new string(word_str));

				msw[token.ID] = pair<stringP, double>(word_strP, 0);
			}
			else
			{

				msw = GlobalVars::lexicon->GetSimilarWords(word_str, lookup_max_ed_dist, lookup_max_cost, errModel, false);

				if (sentence_start && GlobalVars::stringMapper->CharIsUpperCase(word_str[0])) //On the beginning of the sentence both lower case and upper case starting letter should be tried!
				{
					string word_str_lc = word_str;
					word_str_lc[0] = GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(word_str[0]);

					Similar_Words_Map msw_lc = GlobalVars::lexicon->GetSimilarWords(word_str_lc, lookup_max_ed_dist, lookup_max_cost, errModel, false);

					for (Similar_Words_Map::iterator it = msw_lc.begin(); it != msw_lc.end(); it++)
					{
						//word form of founded suggestion should be capitalized!
						if (GlobalVars::stringMapper->CharIsLowerCase(it->second.first->operator [](0)))
							it->second.first->operator [](0) = GlobalVars::stringMapper->GetUpperCaseForLowerCase_char(it->second.first->operator [](0));

						Similar_Words_Map::iterator fit = msw.find(it->first);
						if (fit == msw.end())
						{
							msw[it->first] = it->second;
						}
						else
						{
							if (it->second.second < msw[it->first].second)
							{
								msw[it->first].second = it->second.second;
								msw[it->first].first = it->second.first;
							}
						}
					}
				}

			}

			return msw;

		}

		Similar_Words_Map SimWordsFinder::Find_basic_ignore_case(string &word_str, bool sentence_start, bool keep_orig_casing, uint32_t lookup_max_ed_dist, double lookup_max_cost)
		{
			Similar_Words_Map msw_ic = GlobalVars::lexicon->GetSimilarWords(word_str, lookup_max_ed_dist, lookup_max_cost, errModel, true);

			if (keep_orig_casing == true)
			{
				for (Similar_Words_Map::iterator it = msw_ic.begin(); it != msw_ic.end(); it++)
				{

					stringP sim_w_strP = it->second.first;

					assert(sim_w_strP->length() == word_str.length());
					for (uint32_t j = 0; j < sim_w_strP->size(); j++)
					{
						if (GlobalVars::stringMapper->CharIsUpperCase(sim_w_strP->operator [](j)) &&
							GlobalVars::stringMapper->CharIsLowerCase(word_str[j]))
						{
							sim_w_strP->operator [](j) = GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(sim_w_strP->operator [](j));
						}

						if (GlobalVars::stringMapper->CharIsLowerCase(sim_w_strP->operator [](j)) &&
							GlobalVars::stringMapper->CharIsUpperCase(word_str[j]))
						{
							sim_w_strP->operator [](j) = GlobalVars::stringMapper->GetUpperCaseForLowerCase_char(sim_w_strP->operator [](j));
						}
					}

				}

			}
			else if (sentence_start && GlobalVars::stringMapper->CharIsUpperCase(word_str[0]))
			{
				for (Similar_Words_Map::iterator it = msw_ic.begin(); it != msw_ic.end(); it++)
				{

					stringP sim_w_strP = it->second.first;

					if (GlobalVars::stringMapper->CharIsLowerCase(sim_w_strP->operator [](0)))
					{
						sim_w_strP->operator [](0) = GlobalVars::stringMapper->GetUpperCaseForLowerCase_char(sim_w_strP->operator [](0));
					}

				}

			}

			return msw_ic;

		}
}