/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "SimWordsFinderDiacritic.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "MorphologyMapper.hpp"
#include "AddDiacriticOOVGueser.hpp"
#include "DecoderLetter.hpp"
#include "ErrorModel.hpp"

namespace ngramchecker {

		SimWordsFinderDiacritic::SimWordsFinderDiacritic(ErrorModelP _errModel, bool _sim_words_search_ignore_case, LetterLMSettings _llms):
			SimWordsFinder(_errModel), perform_similar_words_search_ignore_case(_sim_words_search_ignore_case), llms(_llms)
		{}

		Similar_Words_Map SimWordsFinderDiacritic::Find(Token &token, bool sentence_start)
		{
			string word_str = token.GetWordStringISO();

			Similar_Words_Map msw = Find_basic(token, sentence_start, errModel_max_ed_dist, errModel_max_cost);


			if (msw.empty() && perform_similar_words_search_ignore_case)
			{
				msw = Find_basic_ignore_case(word_str, sentence_start, true, errModel_max_ed_dist, errModel_max_cost);

				if (!MyConstants::isMacOS)
				{
					for (Similar_Words_Map::iterator it = msw.begin(); it != msw.end(); it++)
					{
						it->second.first->append("(ignore_case)");
					}

				}
			}

			if (msw.empty())
			{
				bool isName = GlobalVars::stringMapper->CharIsUpperCase(word_str[0]) && sentence_start == false;

				if (llms.use_letter_lm)
				{
					AddDiacriticOOVGueser gueser = AddDiacriticOOVGueser(llms.decoder_order, llms.change_cost_normal, llms.change_cost_name);

					string decoded = gueser.ResolveWord(word_str, isName);

					if (decoded != word_str && !MyConstants::isMacOS)
					{
						decoded += "(letter_lm)";
					}

					stringP decodedP = stringP(new string(decoded));

					if (isName)
					{
						msw[ GlobalVars::morphologyMapper->GetNameTokenID() ] = pair<stringP, double>(decodedP, errModel->NameCost());
					}
					else
					{
						msw[ GlobalVars::morphologyMapper->GetUnkTokenID() ] = pair<stringP, double>(decodedP, errModel->UnknownWordCost());
					}
				}
				else
				{
					stringP word_strP = stringP(new string(word_str));

					if (isName)
					{
						msw[ GlobalVars::morphologyMapper->GetNameTokenID() ] = pair<stringP, double>(word_strP, errModel->NameCost());
					}
					else
					{
						msw[ GlobalVars::morphologyMapper->GetUnkTokenID() ] = pair<stringP, double>(word_strP, errModel->UnknownWordCost());
					}

				}
			}

			return msw;

		}

	const uint32_t SimWordsFinderDiacritic::errModel_max_ed_dist = 20;
	const double SimWordsFinderDiacritic::errModel_max_cost = 500;
}