/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "SimWordsFinderSpelling.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "MorphologyMapper.hpp"
#include "Lexicon.hpp"
#include "ErrorModel.hpp"
#include "Token.hpp"

namespace ngramchecker {

		SimWordsFinderSpelling::SimWordsFinderSpelling(ErrorModelP _errModel, ErrorModelP _errModel_zero_ed_dist, bool _sim_words_search_ignore_case, uint32_t _max_ed_dist_1round,
				uint32_t _max_ed_dist_2round, double _max_cost_1round, double _max_cost_2round):
					SimWordsFinder(_errModel),
					errModel_zero_ed_dist_search(_errModel_zero_ed_dist),
					perform_similar_words_search_ignore_case(_sim_words_search_ignore_case),
					max_ed_dist_1round(_max_ed_dist_1round),
					max_ed_dist_2round(_max_ed_dist_2round),
					max_cost_1round(_max_cost_1round),
					max_cost_2round(_max_cost_2round)
		{}

		SimWordsFinderP SimWordsFinderSpelling::CreateWithDefaultParams(ErrorModelP _errModel, ErrorModelP _errModel_zero_ed_dist, bool _sim_words_search_ignore_case)
		{
			return SimWordsFinderP(new SimWordsFinderSpelling(_errModel, _errModel_zero_ed_dist, _sim_words_search_ignore_case,
					MyConstants::default_similar_words_lookup_max_ed_dist_1round,
					MyConstants::default_similar_words_lookup_max_ed_dist_2round,
					MyConstants::default_similar_words_lookup_max_cost_1round,
					MyConstants::default_similar_words_lookup_max_cost_2round));
		}

		SimWordsFinderP SimWordsFinderSpelling::CreateWithCustomMaxCosts(ErrorModelP _errModel, ErrorModelP _errModel_zero_ed_dist, bool _sim_words_search_ignore_case, double _max_cost_1round, double _max_cost_2round)
		{
			return SimWordsFinderP(new SimWordsFinderSpelling(_errModel, _errModel_zero_ed_dist, _sim_words_search_ignore_case,
					MyConstants::default_similar_words_lookup_max_ed_dist_1round,
					MyConstants::default_similar_words_lookup_max_ed_dist_2round,
					_max_cost_1round,
					_max_cost_2round));
		}

		Similar_Words_Map SimWordsFinderSpelling::Find(Token &token, bool sentence_start)
		{
			string word_str = token.GetWordStringISO();

			Similar_Words_Map msw = Find_basic(token, sentence_start, max_ed_dist_1round, max_cost_1round);

			if (errModel_zero_ed_dist_search != NULL)
			{
				Similar_Words_Map msw2 = GlobalVars::lexicon->GetSimilarWords(token.GetWordStringISO(), 0, max_cost_1round, errModel_zero_ed_dist_search, false);

				for (Similar_Words_Map::iterator it = msw2.begin(); it != msw2.end(); it++)
				{
					if (msw.find(it->first) == msw.end())
					{
						msw[it->first] = it->second;
					}
					else
					{
						if (it->second.second < msw[it->first].second)
						{
							msw[it->first] = it->second;
						}
					}
				}
			}


			if (msw.empty() && perform_similar_words_search_ignore_case)
			{
				msw = Find_basic_ignore_case(word_str, sentence_start, false, max_ed_dist_1round, max_cost_1round);
			}

			if (msw.empty())
			{
				msw = Find_basic(token, sentence_start, max_ed_dist_2round, max_cost_2round);
			}

			if (msw.empty() && perform_similar_words_search_ignore_case)
			{
				msw = Find_basic_ignore_case(word_str, sentence_start, false, max_ed_dist_2round, max_cost_2round);
			}

			if (msw.empty())
			{
				stringP word_strP = stringP(new string(word_str));

				if (GlobalVars::stringMapper->CharIsLowerCase(word_str[0]) ||
					(GlobalVars::stringMapper->CharIsUpperCase(word_str[0]) && sentence_start))
				{
					msw[ GlobalVars::morphologyMapper->GetUnkTokenID() ] = pair<stringP, double>(word_strP, errModel->UnknownWordCost());
				}
				else
				{
					msw[ GlobalVars::morphologyMapper->GetNameTokenID() ] = pair<stringP, double>(word_strP, errModel->NameCost());
				}
			}

			return msw;
		}

}