/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "Splitter.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"

namespace ngramchecker {

		void Splitter::SplitWordToLetters(string word, vector<string> &tokens, vector<string> &delim_seqs)
		{
			for (uint32_t i = 0; i < word.size(); i++)
			{
				if (i > 0) delim_seqs.push_back("");

				assert(GlobalVars::stringMapper->CharIsLetter(word[i]));

				string s = " ";
				s[0] = word[i];
				tokens.push_back(s);
			}
		}

		void Splitter::SplitSentenceToTokens(string sentence, vector<string> &tokens, vector<string> &delim_seqs)
		{
			sentence = sentence + "\n";
			TokenizerState ts = insideToken;

			uint32_t index = 0;

			while (index < sentence.length() && (!GlobalVars::stringMapper->CharIsLetter(sentence[index])) && (!GlobalVars::stringMapper->CharIsNumber(sentence[index])))
			{
				index++;
			}

			uint32_t last_token_char_index = index;
			uint32_t first_token_char_index = index;

			//assert(GlobalVars::stringMapper->CharIsLetter(sentence[0]) || GlobalVars::stringMapper->CharIsNumber(sentence[0]));

			index++;
			while (index < sentence.length())
			{
				switch (ts) {

				case outsideToken:
					if (GlobalVars::stringMapper->CharIsLetter(sentence[index]) || GlobalVars::stringMapper->CharIsNumber(sentence[index]))
					{
						ts = insideToken;
						delim_seqs.push_back(sentence.substr(last_token_char_index + 1, index - last_token_char_index - 1));
						first_token_char_index = index;
					}
					else if (GlobalVars::stringMapper->CharIsPunctuation(sentence[index]))
					{
						//cerr << "punctuation: " << sentence[index] << endl;
						delim_seqs.push_back(sentence.substr(last_token_char_index + 1, index - last_token_char_index - 1));
						string punct_s = "";
						punct_s += sentence[index];
						tokens.push_back(punct_s);
						last_token_char_index = index;
					}
					break;

				case insideToken:

					if (GlobalVars::stringMapper->CharIsPunctuation(sentence[index]))
					{
						//cerr << "punctuation: " << sentence[index] << endl;
						tokens.push_back(sentence.substr(first_token_char_index, index - first_token_char_index));
						string punct_s = "";
						punct_s += sentence[index];
						tokens.push_back(punct_s);
						delim_seqs.push_back("");
						last_token_char_index = index;
						ts = outsideToken;
					}
					else if ((sentence[index] == ' ' || (! GlobalVars::stringMapper->CharIsKnown(sentence[index]))) && (!GlobalVars::stringMapper->CharIsNumber(sentence[index])) )
					{
						tokens.push_back(sentence.substr(first_token_char_index, index - first_token_char_index));
						assert(index != 0);
						last_token_char_index = index - 1;
						ts = outsideToken;
					}

					break;
				}

				index++;
			}
		}

		void Splitter::SplitTextToSentences(string text, vector<string> &sentences, vector<string> &delim_seqs)
		{
			text = text + "\nA";
			uint32_t index = 0;
			SentenceSplitterState sss = noState;
			uint32_t next_sentence_start = 0;
			uint32_t next_sentence_end_candidate = 0;

			while (index < text.size())
			{
				switch (sss) {

				case noState:
					if (GlobalVars::stringMapper->CharIsLetter(text[index]) || GlobalVars::stringMapper->CharIsNumber(text[index]))
					{
						delim_seqs.push_back(text.substr(0, index));
						next_sentence_start = index;
						sss = insideSentence;
					}
					else
					{
						//cerr << "Not letter: " << text[index] << endl;
					}
					index++;
					break;

				case insideSentence:
					if (text[index] == '.' || text[index] == '!' || text[index] == '?' || text[index] == ':')
					{
						next_sentence_end_candidate = index;
						sss = possibleEnding;
					}
					else if (text[index] == '\n')
					{
						next_sentence_end_candidate = index - 1;
						sss = addToDelimSeq;
					}
					index++;
					break;

				case possibleEnding:
					if ( GlobalVars::stringMapper->CharIsLowerCase(text[index]) )
					{
						sss = insideSentence;
					}
					else if ( GlobalVars::stringMapper->CharIsUpperCase(text[index]) )
					{
						sss = finishSentence;
					}
					else if (text[index] == '\n')
					{
						sss = addToDelimSeq;
					}
					else
					{
						index++;
					}
					break;

				case finishSentence:
					sentences.push_back( text.substr(next_sentence_start, next_sentence_end_candidate - next_sentence_start + 1) );
					delim_seqs.push_back( text.substr(next_sentence_end_candidate + 1, index - next_sentence_end_candidate - 1) );
					next_sentence_start = index;
					sss = insideSentence;
					break;

				case addToDelimSeq:
					if (GlobalVars::stringMapper->CharIsLetter(text[index]) || GlobalVars::stringMapper->CharIsNumber(text[index]) )
					{
						sss = finishSentence;
					}
					else
					{
						index++;
					}

					break;
				}
			}

			string last_ds = delim_seqs[delim_seqs.size() - 1];
			last_ds = last_ds.substr(0, last_ds.length() - 1);
			delim_seqs[delim_seqs.size() - 1] = last_ds;


		}

}