/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "StagePosibility.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "MorphologyMapper.hpp"

namespace ngramchecker {


		double StagePosibility_FormsOnly::EmmisionProbability()
		{
			return emmision_prob;
		}

		string StagePosibility_FormsOnly::ToString()
		{
			return word;
		}

		uint32_t StagePosibility_FormsOnly::UniqueIdentifier()
		{
			return id;
		}

		uint32_t StagePosibility_FormsOnly::FormIdentifier()
		{
			return id;
		}

		bool StagePosibility_FormsOnly::IsOriginal()
		{
			return original;
		}

		bool StagePosibility_FormsOnly::IsUnknown()
		{
			return id == GlobalVars::morphologyMapper->GetUnkTokenID() ||
				   id == GlobalVars::morphologyMapper->GetNameTokenID();
		}

		StagePosibilityP StagePosibility_FormsOnly::SentenceStartSP()
		{
			return StagePosibilityP(new StagePosibility_FormsOnly(GlobalVars::morphologyMapper->GetSentenceStartID(), "<s>", true, 0));
		}

		StagePosibilityP StagePosibility_FormsOnly::SentenceEndSP()
		{
			return StagePosibilityP(new StagePosibility_FormsOnly(GlobalVars::morphologyMapper->GetSentenceEndID(), "</s>", true, 0));
		}

		uint32_t StagePosibility_FormsOnly::ID()
		{
			return id;
		}

		StagePosibility_MultiFactor::StagePosibility_MultiFactor(uint32_t _formID, uint32_t _lemmaID, uint32_t _tagID, string _word, bool _original, double _emmision_cost):
			StagePosibility(MultiFactor), formID(_formID), lemmaID(_lemmaID), tagID(_tagID), word(_word), original(_original)
		{
			 uniq_id = 0;

			 boost::hash_combine(uniq_id, _formID);
			 boost::hash_combine(uniq_id, _lemmaID);
			 boost::hash_combine(uniq_id, _tagID);

			 emmision_prob = _emmision_cost;
		}

		StagePosibilityP StagePosibility_MultiFactor::CreateSP_ZeroEmmisionCost(uint32_t formID, uint32_t lemmaID, uint32_t tagID, string word, bool original)
		{
			return StagePosibilityP(new StagePosibility_MultiFactor(formID, lemmaID, tagID, word, original, 0));
		}

		StagePosibilityP StagePosibility_MultiFactor::CreateSP_ErrorModelCost(uint32_t formID, uint32_t lemmaID, uint32_t tagID, string word, bool original, double error_model_cost)
		{
			return StagePosibilityP(new StagePosibility_MultiFactor(formID, lemmaID, tagID, word, original, error_model_cost));
		}

		StagePosibilityP StagePosibility_MultiFactor::CreateSP_ComplexEmCost(uint32_t formID, uint32_t lemmaID, uint32_t tagID, string word, bool original, double error_model_cost, double lemma_em_cost, double tag_em_cost, double lemma_weight, double tag_weight)
		{
			 double emmision_cost = error_model_cost + lemma_weight * lemma_em_cost +
									 tag_weight * tag_em_cost;

			 //cerr << "error_model_cost: " << error_model_cost << endl <<
			//		 "lemma_weight: " << lemma_weight << ", lemma_em_cost: " << lemma_em_cost <<
			//		 "tag_weight: " << tag_weight << ", tag_em_cost: " << tag_em_cost << endl;

			return StagePosibilityP(new StagePosibility_MultiFactor(formID, lemmaID, tagID, word, original, emmision_cost));
		}


		double StagePosibility_MultiFactor::EmmisionProbability()
		{
			return emmision_prob;
		}

		string StagePosibility_MultiFactor::ToString()
		{
			return word; // + "|" + GlobalVars::stringMapper->InvMapLemma(lemmaID) +
				   //"|" + GlobalVars::stringMapper->InvMapTag(tagID);
		}

		uint32_t StagePosibility_MultiFactor::UniqueIdentifier()
		{
			return uniq_id;
		}

		uint32_t StagePosibility_MultiFactor::FormIdentifier()
		{
			return formID;
		}

		bool StagePosibility_MultiFactor::IsOriginal()
		{
			return original;
		}

		bool StagePosibility_MultiFactor::IsUnknown()
		{
			return formID == GlobalVars::morphologyMapper->GetUnkTokenID() ||
				   formID == GlobalVars::morphologyMapper->GetNameTokenID();

		}

		StagePosibilityP StagePosibility_MultiFactor::SentenceStartSP()
		{
			return StagePosibilityP(new StagePosibility_MultiFactor(GlobalVars::morphologyMapper->GetSentenceStartID(),
																	GlobalVars::morphologyMapper->GetSentenceStartID(),
																	GlobalVars::morphologyMapper->GetSentenceStartID(),
																	"<s>", true, 0.0));
		}

		StagePosibilityP StagePosibility_MultiFactor::SentenceEndSP()
		{
			return StagePosibilityP(new StagePosibility_MultiFactor(GlobalVars::morphologyMapper->GetSentenceEndID(),
																  GlobalVars::morphologyMapper->GetSentenceEndID(),
																  GlobalVars::morphologyMapper->GetSentenceEndID(),
																  "</s>", true, 0.0));
		}

		uint32_t StagePosibility_MultiFactor::FormID()
		{
			return formID;
		}

		uint32_t StagePosibility_MultiFactor::LemmaID()
		{
			return lemmaID;
		}

		uint32_t StagePosibility_MultiFactor::TagID()
		{
			return tagID;
		}

		StagePosibility_MorphoAnal::StagePosibility_MorphoAnal(uint32_t _formID, uint32_t _lemmaID, uint32_t _tagID, double _lemma_emmision_cost, double _tag_emmision_cost, double _lemma_weight, double _tag_weight):
			StagePosibility(MorphoAnal), formID(_formID), lemmaID(_lemmaID), tagID(_tagID)
		{
			 uniq_id = 0;

			 boost::hash_combine(uniq_id, _formID);
			 boost::hash_combine(uniq_id, _lemmaID);
			 boost::hash_combine(uniq_id, _tagID);

			 emmision_prob = _lemma_weight * _lemma_emmision_cost +
							  _tag_weight * _tag_emmision_cost;

			 assert(emmision_prob >= 0);
		}

		double StagePosibility_MorphoAnal::EmmisionProbability()
		{
			return emmision_prob;
		}

		string StagePosibility_MorphoAnal::ToString()
		{
			string ret = "";
			if (output_form)
			{
				ret += GlobalVars::stringMapper->InvMapForm(formID);
			}

			if (output_lemma)
			{
				if (ret != "") ret += "|";
				ret += GlobalVars::stringMapper->InvMapLemma(lemmaID);
			}

			if (output_tag)
			{
				if (ret != "") ret += "|";
				ret += GlobalVars::stringMapper->InvMapTag(tagID);
			}

			return ret;
		}

		uint32_t StagePosibility_MorphoAnal::UniqueIdentifier()
		{
			return uniq_id;
		}

		uint32_t StagePosibility_MorphoAnal::FormIdentifier()
		{
			return formID;
		}

		bool StagePosibility_MorphoAnal::IsOriginal()
		{
			return false;
		}

		bool StagePosibility_MorphoAnal::IsUnknown()
		{
			return false;
		}

		StagePosibilityP StagePosibility_MorphoAnal::SentenceStartSP()
		{
			return StagePosibilityP(new StagePosibility_MorphoAnal(GlobalVars::morphologyMapper->GetSentenceStartID(),
																	GlobalVars::morphologyMapper->GetSentenceStartID(),
																	GlobalVars::morphologyMapper->GetSentenceStartID(),
																	0.0, 0.0, 0.0, 0.0));
		}

		StagePosibilityP StagePosibility_MorphoAnal::SentenceEndSP()
		{
			return StagePosibilityP(new StagePosibility_MorphoAnal(GlobalVars::morphologyMapper->GetSentenceEndID(),
																  GlobalVars::morphologyMapper->GetSentenceEndID(),
																  GlobalVars::morphologyMapper->GetSentenceEndID(),
																  0.0, 0.0, 0.0, 0.0));
		}

		uint32_t StagePosibility_MorphoAnal::FormID()
		{
			return formID;
		}

		uint32_t StagePosibility_MorphoAnal::LemmaID()
		{
			return lemmaID;
		}

		uint32_t StagePosibility_MorphoAnal::TagID()
		{
			return tagID;
		}

		StagePosibility_Letter::StagePosibility_Letter(uint32_t _letter, double _emmision_prob):
			StagePosibility(Letter), letter_ID(_letter), emmision_prob(_emmision_prob)
		{
			str_form = " ";
			str_form[0] = GlobalVars::stringMapper->InvMapChar(_letter);

			if (str_form == ".")
				str_form = "<s>";

			if (str_form == ",")
				str_form = "</s>";
		}

		double StagePosibility_Letter::EmmisionProbability()
		{
			return emmision_prob;
		}

		string StagePosibility_Letter::ToString()
		{
			return str_form;
		}

		bool StagePosibility_Letter::IsOriginal()
		{
			return false;
		}

		uint32_t StagePosibility_Letter::UniqueIdentifier()
		{
			return letter_ID;
		}

		uint32_t StagePosibility_Letter::FormIdentifier()
		{
			return letter_ID;
		}

		bool StagePosibility_Letter::IsUnknown()
		{
			return false;
		}

		uint32_t StagePosibility_Letter::LetterID()
		{
			return letter_ID;
		}

		StagePosibilityP StagePosibility_Letter::SentenceStartSP()
		{
			return StagePosibilityP(new StagePosibility_Letter(GlobalVars::stringMapper->MapChar('.'), 0));
		}

		StagePosibilityP StagePosibility_Letter::SentenceEndSP()
		{
			return StagePosibilityP(new StagePosibility_Letter(GlobalVars::stringMapper->MapChar(','), 0));
		}
}