/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */


#ifndef STRINGMAPPER_HPP_
#define STRINGMAPPER_HPP_

#include "StdAfx.h"

namespace ngramchecker {

	class StringMapper {
	private:

		static StringMapper stringMapper;

		map<char, uint32_t> charmap;
		map<uint32_t, char> invcharmap;

		set<char> vocal_set;

		uint32_t punctuation_start_index;
		uint32_t punctuation_end_index;

		uint32_t number_start_index;
		uint32_t number_end_index;

		boost::shared_ptr<map<uint32_t, string> > form_inv_map;
		boost::shared_ptr<map<string, uint32_t> > form_map;
		boost::shared_ptr<map<uint32_t, string> > lemma_inv_map;
		boost::shared_ptr<map<string, uint32_t> > lemma_map;
		boost::shared_ptr<map<uint32_t, string> > tag_inv_map;
		boost::shared_ptr<map<string, uint32_t> > tag_map;

		uint32_t num_forms;
		uint32_t num_lemmas;
		uint32_t num_tags;

		uint32_t capitals_start_index;

		map<pair<char, char>, char> utf8_iso_map;
		map<char, pair<char, char> > iso_utf8_map;

		map<uint32_t, vector<uint32_t> > diac_map;
		map<uint32_t, uint32_t> diac_inv_map;


		 void initFormMaps();

		 void initLemmaMaps();

		 void initTagMaps();

			bool SymbolIsPunctuation(uint32_t character);

			bool SymbolIsLetter(uint32_t character);

			uint32_t GetLowerCaseForUpperCase_symbol(uint32_t letterIndex);

			uint32_t GetUpperCaseForLowerCase_symbol(uint32_t letterIndex);

			bool LetterIsDiacritic(uint32_t letter);

			uint32_t GetBasicLetterForDiacriticLetter(uint32_t diacritic_letter);

			bool LetterHasDiacriticVariants(uint32_t letter);

	public:

			StringMapper();


		bool CharIsNumber(char ch);

		bool WordIsNumber(const string &word);

		bool CharIsPunctuation(char ch);
		
		bool CharIsVocal(char ch);

		bool CharIsKnown(char ch);

		bool CharIsLetter(char ch);

		bool CharIsLowerCase(char ch);

		bool CharIsUpperCase(char ch);

		 uint32_t NumForms();

		uint32_t NumLemmas();

		uint32_t NumTags();

		string InvMapForm(uint32_t id);

		string InvMapLemma(uint32_t id);

		string InvMapTag(uint32_t id);

		uint32_t MapForm(string form);

		bool ContainsLemma(string lemma);

		uint32_t MapLemma(string lemma);

		uint32_t MapTag(string tag);

		uint32_t MapChar(char ch);

		uint32_t PunctuationIndex_char(char character);


		char InvMapChar(uint32_t val);

		char GetLowerCaseForUpperCase_char(char character);

		char GetUpperCaseForLowerCase_char(char character);

		string InvMap(vector<uint32_t> &vec);

		vector<uint32_t> MapString(string s);

		uint32_t NumLetters();

		string ConvertToISO(string input);

		string ConvertBackFromISO(string iso_str);

		string RemoveDiacritics_ISO(string text);

		bool CaseInsensitiveIsEqual(string str1, string str2);

	};

}

#endif /* STRINGMAPPER_HPP_ */
