/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef TEXTCHECKINGRESULT_HPP_
#define TEXTCHECKINGRESULT_HPP_

#include "StdAfx.h"

namespace ngramchecker {

	enum TextCheckingResultType { grammar, spelling };

	class TextCheckingResult {
	public:
		uint32_t range_from;
		uint32_t range_length;
		vector<string> suggestions;
		TextCheckingResultType type;
		string orig_word;
	
		TextCheckingResult(uint32_t _range_from, uint32_t _range_length, vector<string> &_suggestions, TextCheckingResultType _type, string _orig_word);

		virtual boost::shared_ptr<TextCheckingResult> makeShiftedCopy(uint32_t shift) = 0;
	
		virtual string ToString() = 0;
	
	};

	class SpellingCheckingResult : public TextCheckingResult {
	public:
	
		SpellingCheckingResult(uint32_t _range_from, uint32_t _range_length, vector<string> &_suggestions, string _orig_word);

		virtual boost::shared_ptr<TextCheckingResult> makeShiftedCopy(uint32_t shift);
	
		virtual string ToString();

	};

	class GrammarCheckingResult : public TextCheckingResult {
	public:
		string error_description;
	
		GrammarCheckingResult(uint32_t _range_from, uint32_t _range_until, vector<string> &_suggestions, string _error_description, string _orig_word);

		virtual boost::shared_ptr<TextCheckingResult> makeShiftedCopy(uint32_t shift);
	
		virtual string ToString();

	};

	SP_DEF(GrammarCheckingResult);
	SP_DEF(SpellingCheckingResult);

	SP_DEF(TextCheckingResult);
	typedef boost::shared_ptr<vector<boost::shared_ptr<TextCheckingResult> > > vectorP_TextCheckingResultP;

}

#endif /* TEXTCHECKINGRESULT_HPP_ */
