/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */
#include "TextCheckingServer.hpp"
#include "GlobalVars.hpp"
#include "LMServer.hpp"
#include "SimWordsFinderSpelling.hpp"
#include "SimWordsFinderDiacritic.hpp"
#include "StringMapper.hpp"
#include "DecoderMultiFactor.hpp"
#include "DecoderLetter.hpp"
#include "Spellchecker.hpp"
#include "ErrorModel.hpp"
#include "ErrorModelAbbreviation.hpp"
#include "ErrorModelDiacriticsSimple.hpp"
#include "ErrorModelIgnoreDiacriticsTypo.hpp"
#include "ErrorModelTypo.hpp"
#include "ErrorModelTypo_Dia_and_Cap_Zero_Dist.hpp"
#include "DiacriticComplement.hpp"
#include "LemmaSigns.hpp"
#include "Splitter.hpp"
#include "TextCheckingResult.hpp"

namespace ngramchecker {

	vector<string> TextCheckingServer::S_W_M_to_V_S(Similar_Words_Map &swm)
	{
		vector<pair<string, double> > word_cost_vec;

		for (Similar_Words_Map::iterator it = swm.begin(); it != swm.end(); it++)
		{
			uint32_t formID = it->first;
			double cost = it->second.second;
			string sim_word = *(it->second.first);


			cost += GlobalVars::lmServer->GetContextFreeCost(formID);


			word_cost_vec.push_back(pair<string, double>(sim_word, cost));

		}

		std::sort(word_cost_vec.begin(), word_cost_vec.end(), Pair_string_double_comparer());

		vector<string> ret;
		for (uint32_t i = 0; i < word_cost_vec.size(); i++)
		{
			ret.push_back(word_cost_vec[i].first);
		}

		return ret;

	}


	vector<string> TextCheckingServer::GetContextFreeSuggestions_ErrorModel(string iso_word, ErrorModelP errMod, ErrorModelP errMod_zero_ed_dist)
	{

		SimWordsFinderP swfs = SimWordsFinderSpelling::CreateWithDefaultParams(errMod, errMod_zero_ed_dist, false);

		Token tok = Token(iso_word);

		Similar_Words_Map swm = swfs->Find(tok, true);

		/*Similar_Words_Map swm = GlobalVars::lexicon->GetSimilarWords(iso_word, default_similar_words_lookup_max_ed_dist_1round, default_similar_words_lookup_max_cost_1round, errMod, false);

		if (swm.empty())
		{
			swm = GlobalVars::lexicon->GetSimilarWords(iso_word, default_similar_words_lookup_max_ed_dist_2round, default_similar_words_lookup_max_cost_2round, errMod, false);
		}*/

		vector<string> ret = S_W_M_to_V_S(swm);

		return ret;
	}

	void TextCheckingServer::FindMisspelledWordIgnoreDiacritics(string &iso_text, uint32_t &range_from, uint32_t &range_length)
	{
		bool insideWord = false;

		uint32_t index = 0;
		uint32_t word_start_index = 0;

		while (index < iso_text.length())
		{
			if (insideWord == false)
			{
				if (GlobalVars::stringMapper->CharIsLetter(iso_text[index]))
				{
					insideWord = true;
					word_start_index = index;
				}
			}
			else
			{
				if (!GlobalVars::stringMapper->CharIsLetter(iso_text[index]))
				{
					string word_str;

					for (uint32_t i = word_start_index; i < index; i++)
					{
						word_str.push_back(iso_text[i]);
					}

					bool ok = true;

					SimWordsFinderDiacritic swfd = SimWordsFinderDiacritic(GlobalVars::errorModelDiacriticsSimpleP, true, LetterLMSettings(false, 0, 0, 0));

					Token tok = Token(word_str);
					Similar_Words_Map swm = swfd.Find(tok, true);

					if (swm.empty() || swm.begin()->second.second > 0)
					{
						ok = false;
					}

					/*Similar_Words_Map swm = GlobalVars::lexicon->GetSimilarWords(word_str, 1000, 1000, GlobalVars::errorModelDiacriticsSimpleP, false);

					if (! swm.empty())
						ok = true;

					if (ok == false) {
						if (GlobalVars::stringMapper->CharIsLowerCase(word_str[0]))
						{
							word_str[0] = GlobalVars::stringMapper->GetUpperCaseForLowerCase_char(word_str[0]);
						}
						else
						{
							word_str[0] = GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(word_str[0]);
						}

						swm = GlobalVars::lexicon->GetSimilarWords(word_str, 1000, 1000, GlobalVars::errorModelDiacriticsSimpleP, false);

						if (! swm.empty())
							ok = true;
					}*/

					if (ok == false) {
						range_from = word_start_index;
						range_length = index - word_start_index;
						return;
					}
					else {
						//cerr << "word is ok: " << stringMapper.InvMap(word_vec) << endl;
					}

					insideWord = false;

				}
			}


			index++;
		}

		range_from = 0;
		range_length = 0;
		return;
	}

	void TextCheckingServer::FindMisspelledWordWithDiacritics(string &iso_text, uint32_t &range_from, uint32_t &range_length)
	{
		bool insideWord = false;

		uint32_t index = 0;
		uint32_t word_start_index = 0;

		while (index < iso_text.length())
		{
			if (insideWord == false)
			{
				if (GlobalVars::stringMapper->CharIsLetter(iso_text[index]))
				{
					insideWord = true;
					word_start_index = index;
				}
			}
			else
			{
				if (!GlobalVars::stringMapper->CharIsLetter(iso_text[index]))
				{
					string word_str;

					for (uint32_t i = word_start_index; i < index; i++)
					{
						word_str.push_back(iso_text[i]);
					}

					//cerr << "word_str = " << word_str << endl;
					bool ok = true;

					SimWordsFinderP swfs = SimWordsFinderSpelling::CreateWithCustomMaxCosts(GlobalVars::errorModelTypoP, ErrorModelP(), true, 0, 0);

					Token tok = Token(word_str);
					Similar_Words_Map swm = swfs->Find(tok, true);

					if (swm.empty() || swm.begin()->second.second > 0)
					{
						ok = false;
					}

					/*for (Similar_Words_Map::iterator it = swm.begin(); it != swm.end(); it++)
					{
						cerr << *(it->second.first) << " ~ " << it->second.second << endl;
					}*/


					/*if (GlobalVars::lexicon->GetWordID(word_str) != -1 ) {
						ok = true;
					}

					if (ok == false) {
						if (GlobalVars::stringMapper->CharIsLowerCase(word_str[0]))
						{
							word_str[0] = GlobalVars::stringMapper->GetUpperCaseForLowerCase_char(word_str[0]);
						}
						else
						{
							word_str[0] = GlobalVars::stringMapper->GetLowerCaseForUpperCase_char(word_str[0]);
						}

						if (GlobalVars::lexicon->GetWordID(word_str) != -1) {
							ok = true;
						}
					}*/

					if (ok == false) {
						range_from = word_start_index;
						range_length = index - word_start_index;
						return;
					}

					insideWord = false;

				}
			}


			index++;
		}

		range_from = 0;
		range_length = 0;
		return;
	}

	void TextCheckingServer::SetDiacriticSensitivity(bool isSensitive)
	{
		diacritic_sensitive = isSensitive;
	}

	bool TextCheckingServer::GetDiacriticSensitivity()
	{
		return diacritic_sensitive;
	}


	vector<string> TextCheckingServer::PerformServerCommand(const string &command)
	{
		vector<string> ret;

		if (command == "?")
		{
			ret.push_back("michalisekSpell help:");
			ret.push_back(" ");
			ret.push_back("Nastavení diakritiky:");
			ret.push_back("!diacritic=on   --- zapne režim kontroly s diakritikou");
			ret.push_back("!diacritic=off  --- zapne režim kontroly bez diakritiky");
			ret.push_back(" ");
			ret.push_back("Nastavení stylových příznaků:");
			ret.push_back("!signs=[seznam příznaků]  --- Nastaví povolené příznkaky" );
			ret.push_back("!signs+[seznam příznaků]  --- Přidá uvedené příznaky mezi povolené");
			ret.push_back("!signs-[seznam příznaků]  --- Odebere uvedené příznaky ze seznamu povolených");
			ret.push_back(" ");
			ret.push_back("Platné příznaky:");

			vector<string> all_signs = GlobalVars::lemmaSigns->GetSignDescribtions();
			ret.insert(ret.end(), all_signs.begin(), all_signs.end());

		}
		else if (command == "!")
		{
			ret.push_back("michalisekSpell - Stav serveru:");

			if (diacritic_sensitive)
				ret.push_back("Režim: kontrola s diakritikou");
			else
				ret.push_back("Režim: kontrola bez diakritiky");

			ret.push_back(" ");
			vector<string> ls = GlobalVars::lemmaSigns->GetCurrentStateDescribtion();
			ret.insert(ret.end(), ls.begin(), ls.end());
		}
		else if (command == "!diacritic=on")
		{
			if (GetDiacriticSensitivity() == false)
			{
				SetDiacriticSensitivity(true);
				ret.push_back("Režim kontroly s diakritikou byl právě zapnut!");
			}
			else
				ret.push_back("Žádná změna!");
				ret.push_back("Režim kontroly s diakritikou je již aktivní!");
		}
		else if (command == "!diacritic=off")
		{
			if (GetDiacriticSensitivity() == true)
			{
				SetDiacriticSensitivity(false);
				ret.push_back("Režim kontroly bez diakritikou byl právě zapnut!");
			}
			else
				ret.push_back("Žádná změna!");
				ret.push_back("Režim kontroly bez diakritikou je již aktivní!");

		}
		else if (command.substr(0, 7) == "!signs=")
		{
			GlobalVars::lemmaSigns->SetConditionsSignature(command.substr(7, command.length() - 7));

			ret.push_back("Povolené příznaky nastaveny!");
			ret.push_back(" ");
			ret.push_back("Nové nastavení:");

			vector<string> ls = GlobalVars::lemmaSigns->GetCurrentStateDescribtion();
			ret.insert(ret.end(), ls.begin(), ls.end());
		}
		else if (command.substr(0, 7) == "!signs+")
		{
			GlobalVars::lemmaSigns->AllowSigns(command.substr(7, command.length() - 7));

			ret.push_back("Povolené příznaky nastaveny!");
			ret.push_back(" ");
			ret.push_back("Nové nastavení:");

			vector<string> ls = GlobalVars::lemmaSigns->GetCurrentStateDescribtion();
			ret.insert(ret.end(), ls.begin(), ls.end());
		}
		else if (command.substr(0, 7) == "!signs-")
		{
			GlobalVars::lemmaSigns->DisallowSigns(command.substr(7, command.length() - 7));

			ret.push_back("Povolené příznaky nastaveny!");
			ret.push_back(" ");
			ret.push_back("Nové nastavení:");

			vector<string> ls = GlobalVars::lemmaSigns->GetCurrentStateDescribtion();
			ret.insert(ret.end(), ls.begin(), ls.end());
		}
		else
		{
			ret.push_back("Neplatný příkaz!");
			ret.push_back("Pro zobrazení nápovědy použijte: ?");
		}

		return ret;
	}

	void TextCheckingServer::FindMisspelledWord(string &text, uint32_t &range_from, uint32_t &range_length)
	{
		if (! diacritic_sensitive)
			FindMisspelledWordIgnoreDiacritics(text, range_from, range_length);
		else
			FindMisspelledWordWithDiacritics(text, range_from, range_length);
	}



	vector<string> TextCheckingServer::GetContextFreeSuggestions(string word)
	{
		for (uint32_t i = 0; i < word.length(); i++)
		{
			if (! GlobalVars::stringMapper->CharIsKnown(word[i]))
			{
				vector<string> ret;
				return ret;
			}
		}

		if (diacritic_sensitive)
		{
			return GetContextFreeSuggestions_ErrorModel(word, GlobalVars::errorModelTypoDiaCapZeroDistP, ErrorModelP());
		}
		else
		{
			return GetContextFreeSuggestions_ErrorModel(word, GlobalVars::errorModelIgnoreDiacriticsTypoDiaCapZeroDistP, ErrorModelP());
		}
	}

	vector<string> TextCheckingServer::GetSuggestions(string word)
	{

		if (word[0] == '!' || word[0] == '?')
		{
			return PerformServerCommand(word);
		}

		if (misspelled_words_suggestions.IsCashed(word))
		{
			vector<string> ret = misspelled_words_suggestions.GetCashedValue(word);
			return ret;
		}
		else
		{
			return GetContextFreeSuggestions(word);
		}
	}

	string TextCheckingServer::AddDiacritics(string iso_text)
	{
		string ret;
		vector<string> sentences;
		vector<string> delim_seqs;

		Splitter::SplitTextToSentences(iso_text, sentences, delim_seqs);
		ret = delim_seqs[0];


		SimWordsFinderP swf = SimWordsFinderDiacriticP(new SimWordsFinderDiacritic(GlobalVars::errorModelDiacriticsSimpleP, MyConstants::diacritic_completion_ignore_case, LetterLMSettings(MyConstants::use_letter_lm, MyConstants::letter_decoder_order, 0.4, 1.2)));

		for (uint32_t i = 0; i < sentences.size(); i++)
		{

			DecoderBaseP dmf = DecoderBaseP(new DecoderMultiFactor(MyConstants::form_order, MyConstants::lemma_order,
					MyConstants::tag_order, swf, MyConstants::form_weight, MyConstants::lemma_weight, MyConstants::tag_weight));

			DiacriticComplement dc = DiacriticComplement(dmf);

			ret += dc.AddDiacriticToSentence(sentences[i]);
			ret += delim_seqs[i + 1];
		}

		return ret;
	}

	string TextCheckingServer::RemoveDiacritics(const string &iso_text)
	{
		return GlobalVars::stringMapper->RemoveDiacritics_ISO(iso_text);
	}

	string TextCheckingServer::AutoCorrect(const string &iso_text)
	{

		string ret;
		vector<string> sentences;
		vector<string> delim_seqs;

		Splitter::SplitTextToSentences(iso_text, sentences, delim_seqs);
		ret = delim_seqs[0];

		SimWordsFinderP swf;

		if (diacritic_sensitive)
		{
			 swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelTypoDiaCapZeroDistP/*GlobalVars::errorModelTypoP*/, ErrorModelP()/*GlobalVars::errorModelTypoDiaCapZeroDistP*/, false);
		}
		else
		{
			 swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelIgnoreDiacriticsTypoDiaCapZeroDistP, ErrorModelP(), false);
		}

		for (uint32_t i = 0; i < sentences.size(); i++)
		{
			DecoderBaseP decoder = DecoderBaseP(new DecoderMultiFactor(MyConstants::form_order, MyConstants::lemma_order,
					MyConstants::tag_order, swf, MyConstants::form_weight, MyConstants::lemma_weight, MyConstants::tag_weight));

			Spellchecker spellchecker = Spellchecker(decoder);

			ret += spellchecker.CheckSentence(sentences[i]);
			ret += delim_seqs[i + 1];
		}

		return ret;

	}
	
	string TextCheckingServer::CompleteAbbreviatedText(const string &iso_text)
	{

		string ret;
		vector<string> sentences;
		vector<string> delim_seqs;

		Splitter::SplitTextToSentences(iso_text, sentences, delim_seqs);
		ret = delim_seqs[0];

		SimWordsFinderP swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelAbbreviationP, ErrorModelP(), false);

		for (uint32_t i = 0; i < sentences.size(); i++)
		{
			DecoderBaseP decoder = DecoderBaseP(new DecoderMultiFactor(MyConstants::form_order, MyConstants::lemma_order,
					MyConstants::tag_order, swf, MyConstants::form_weight, MyConstants::lemma_weight, MyConstants::tag_weight));

			Spellchecker spellchecker = Spellchecker(decoder);

			ret += spellchecker.CheckSentence(sentences[i]);
			ret += delim_seqs[i + 1];
		}

		return ret;

	}
	
	string TextCheckingServer::AbbreviateText(const string &iso_text)
	{
		char* out = new char[iso_text.length()];
		
		uint32_t out_index = 0;
		
		for (uint32_t i = 0; i < iso_text.length(); i++)
		{
			if ( (GlobalVars::stringMapper->CharIsVocal(iso_text[i]) || (i > 0 && iso_text[i] == iso_text[i - 1]) ) && 
				( (i > 0 && GlobalVars::stringMapper->CharIsLetter(iso_text[i - 1])) ||
				  (i < iso_text.length() - 1 && GlobalVars::stringMapper->CharIsLetter(iso_text[i + 1])) ) )
			{
				continue; //skip the letter
			}
			
			out[out_index] = iso_text[i];
			out_index++;
		}
		
		out[out_index] = 0;
		
		string ret_string = out;
		
		delete[] out;
		
		return ret_string;
	}
	
	string TextCheckingServer::SpellCheck_command_line_mode(const string &iso_text)
	{

		string ret;
		vector<string> sentences;
		vector<string> delim_seqs;

		Splitter::SplitTextToSentences(iso_text, sentences, delim_seqs);
		ret = delim_seqs[0];

		SimWordsFinderP swf;

		if (diacritic_sensitive)
		{
			 swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelTypoDiaCapZeroDistP/*GlobalVars::errorModelTypoP*/, ErrorModelP()/*GlobalVars::errorModelTypoDiaCapZeroDistP*/, false);
		}
		else
		{
			 swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelIgnoreDiacriticsTypoDiaCapZeroDistP, ErrorModelP(), false);
		}

		for (uint32_t i = 0; i < sentences.size(); i++)
		{
			DecoderBaseP decoder = DecoderBaseP(new DecoderMultiFactor(MyConstants::form_order, MyConstants::lemma_order,
					MyConstants::tag_order, swf, MyConstants::form_weight, MyConstants::lemma_weight, MyConstants::tag_weight));

			Spellchecker spellchecker = Spellchecker(decoder);

			ret += spellchecker.command_line_mode(sentences[i], MyConstants::default_num_suggestions);
			ret += delim_seqs[i + 1];
		}

		return ret;

	}

	string TextCheckingServer::SpellChecking_evaluate(const string &iso_text)
	{

		string ret;
		vector<string> sentences;
		vector<string> delim_seqs;

		Splitter::SplitTextToSentences(iso_text, sentences, delim_seqs);
		ret = delim_seqs[0];

		SimWordsFinderP swf;

		if (diacritic_sensitive)
		{
			 swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelTypoDiaCapZeroDistP/*GlobalVars::errorModelTypoP*/, ErrorModelP()/*GlobalVars::errorModelTypoDiaCapZeroDistP*/, false);
		}
		else
		{
			 swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelIgnoreDiacriticsTypoDiaCapZeroDistP, ErrorModelP(), false);
		}

		for (uint32_t i = 0; i < sentences.size(); i++)
		{
			DecoderBaseP decoder = DecoderBaseP(new DecoderMultiFactor(MyConstants::form_order, MyConstants::lemma_order,
					MyConstants::tag_order, swf, MyConstants::form_weight, MyConstants::lemma_weight, MyConstants::tag_weight));

			Spellchecker spellchecker = Spellchecker(decoder);

			ret += spellchecker.DecodeEvaluation(sentences[i], MyConstants::default_num_suggestions);
			ret += delim_seqs[i + 1];
		}

		return ret;

	}


	vector<TextCheckingResultP> TextCheckingServer::ProcessTextSingleThread(string iso_text, uint32_t &range_from, uint32_t &range_length)
	{

		vector<TextCheckingResultP> ret;

		vector<string> sentences;
		vector<string> delim_seqs;

		Splitter::SplitTextToSentences(iso_text, sentences, delim_seqs);

		if (sentences.size() == 0)
		{
			range_from = 0;
			range_length = 0;
			return ret;
		}

		char fs_lch = sentences[0][sentences[0].length() - 1];

		if (fs_lch != '.' && fs_lch != '!' && fs_lch != '?' && fs_lch != ':')
		{
			range_from = 0;
			range_length = 0;
			return ret;
		}

		range_from = delim_seqs[0].length();
		range_length = sentences[0].length();


		string sent = sentences[0];



		if (checking_results_cash.IsCashed(sent))
		{
			ret = checking_results_cash.GetCashedValue(sent);

		}
		else
		{

			SimWordsFinderP swf;

			if (diacritic_sensitive)
			{
				 swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelTypoP, GlobalVars::errorModelTypoDiaCapZeroDistP, false);
			}
			else
			{
				 swf = SimWordsFinderSpelling::CreateWithDefaultParams(GlobalVars::errorModelIgnoreDiacriticsTypoDiaCapZeroDistP, ErrorModelP(), false);
			}

			DecoderBaseP decoder = DecoderBaseP(new DecoderMultiFactor(MyConstants::form_order, MyConstants::lemma_order,
					MyConstants::tag_order, swf, MyConstants::form_weight, MyConstants::lemma_weight, MyConstants::tag_weight));

			Spellchecker spellchecker = Spellchecker(decoder);

			ret = spellchecker.GetCheckingResults(sent);

			//lmServer.PrintStats();
			//lmServer.ClearCash();

			checking_results_cash.StoreValueForKey(sent, ret);

		}

		for (uint32_t i = 0; i < ret.size(); i++)
		{
			TextCheckingResultP tchrp = ret[i];

			if (tchrp->type == spelling)
			{
				//misspelled_words_suggestions[tchrp->orig_word] = tchrp->suggestions;
				misspelled_words_suggestions.StoreValueForKey(sentences[0].substr(tchrp->range_from, tchrp->range_length), tchrp->suggestions);
			}
		}

		return ret;

	}
}
