/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef TEXTCHECKINGSERVER_HPP_
#define TEXTCHECKINGSERVER_HPP_

#include "StdAfx.h"
#include "MyCash.hpp"

namespace ngramchecker {

	class TextCheckingResult;
	SP_DEF(TextCheckingResult);

	class ErrorModel;
	SP_DEF(ErrorModel);

struct stringP_comparer : less<stringP> {
	bool operator()(const stringP& val1, const stringP& val2)
	{
		return *val1 < *val2;
	}
};

struct Pair_string_double_comparer : less<pair<string, double> > {
	bool operator()(const pair<string, double> &val1, const pair<string, double> &val2)
	{
		return val1.second < val2.second;
	}
};

//typedef map<stringP, list<stringP>::iterator, stringP_comparer > string_list_map;

typedef MyCash<string, vector<TextCheckingResultP> > MyCash_TextCheckingResults;
typedef MyCash<string, vector<string> > MyCash_Suggestions;

class TextCheckingServer {
private:
	MyCash_TextCheckingResults checking_results_cash;

	MyCash_Suggestions misspelled_words_suggestions;

	bool diacritic_sensitive;

	vector<string> S_W_M_to_V_S(Similar_Words_Map &swm);

	vector<string> GetContextFreeSuggestions_ErrorModel(string iso_word, ErrorModelP errMod, ErrorModelP errMod_zero_ed_dist);

	void FindMisspelledWordIgnoreDiacritics(string &iso_text, uint32_t &range_from, uint32_t &range_length);

	void FindMisspelledWordWithDiacritics(string &iso_text, uint32_t &range_from, uint32_t &range_length);

public:

	void SetDiacriticSensitivity(bool isSensitive);

	bool GetDiacriticSensitivity();

	vector<string> PerformServerCommand(const string &command);

	TextCheckingServer():
		checking_results_cash(MyConstants::default_checking_results_cash_capacity),
		misspelled_words_suggestions(MyConstants::default_suggestions_cash_capacity),
		diacritic_sensitive(true)
	{}


	void FindMisspelledWord(string &text, uint32_t &range_from, uint32_t &range_length);

	vector<string> GetContextFreeSuggestions(string word);

	vector<string> GetSuggestions(string word);

	string AddDiacritics(string iso_text);

	string RemoveDiacritics(const string &iso_text);

	string AutoCorrect(const string &iso_text);
	
	string CompleteAbbreviatedText(const string &iso_text);
	
	string AbbreviateText(const string &iso_text);
	
	string SpellCheck_command_line_mode(const string &iso_text);

	string SpellChecking_evaluate(const string &iso_text);

	vector<TextCheckingResultP> ProcessTextSingleThread(string iso_text, uint32_t &range_from, uint32_t &range_length);


};

}

#endif /* TEXTCHECKINGSERVER_HPP_ */
