/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "Token.hpp"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "MorphologyMapper.hpp"
#include "Lexicon.hpp"
#include "utils.hpp"

namespace ngramchecker {

		Token::Token(string _word_iso)
		{
			word_iso = _word_iso;

			if (word_iso.length() == 1 && GlobalVars::stringMapper->CharIsPunctuation(word_iso[0]))
			{
				ID = GlobalVars::morphologyMapper->PunctuationUniversalIndex_char(word_iso[0]);
			}
			else if (GlobalVars::stringMapper->WordIsNumber(word_iso))
			{
				ID = GlobalVars::morphologyMapper->GetNumberTokenID();
			}
			else
			{
				int _id = GlobalVars::lexicon->GetWordID(word_iso);

				if (_id == -1)
				{
					ID = GlobalVars::morphologyMapper->GetUnkTokenID(); //TODO: Check this, what about names?
				}
				else
				{
					ID = _id;
				}
			}
		}

		Token::Token(uint32_t letter_ID)
		{
			word_iso = " ";
		
			word_iso[0] = GlobalVars::stringMapper->InvMapChar(letter_ID);
			ID = letter_ID;
			//assert(GlobalVars::stringMapper->CharIsLetter(word_iso[0]));

		}

		bool Token::IsUnknown()
		{
			//cerr << "!!! Unknown token: " << word_iso << endl;
			return ID == GlobalVars::morphologyMapper->GetUnkTokenID() ||
					ID == GlobalVars::morphologyMapper->GetNameTokenID();
		}


		string Token::GetWordStringISO()
		{
			return word_iso;
		}

		string Token::GetWordStringUTF8()
		{
			return GlobalVars::stringMapper->ConvertBackFromISO(GetWordStringISO());
		}

		vector<uint32_t> Token::GetWordVector()
		{
			return GlobalVars::stringMapper->MapString(word_iso);
		}
}