/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */


#ifndef TRANSITIONCOSTCOMPUTATION_HPP_
#define TRANSITIONCOSTCOMPUTATION_HPP_

#include "StdAfx.h"

namespace ngramchecker {

	class ViterbiState;
	SP_DEF(ViterbiState);

	class StagePosibility;
	SP_DEF(StagePosibility);

class TransitionCostComputation {
public:

	virtual double ComputeTransitionCost(ViterbiStateP state, StagePosibilityP next) = 0;

	virtual double ComputeTransitionCostSPSequence(vector<StagePosibilityP> &sp_vec, uint32_t start_index, uint32_t end_index) = 0;
};

SP_DEF(TransitionCostComputation);

class TransitionCostComputation_FormsOnly : public TransitionCostComputation {
private:
	double form_weight;
public:

	TransitionCostComputation_FormsOnly(double _form_weight): form_weight(_form_weight) {}

	virtual double ComputeTransitionCost(ViterbiStateP state, StagePosibilityP next);

	virtual double ComputeTransitionCostSPSequence(vector<StagePosibilityP> &sp_vec, uint32_t start_index, uint32_t end_index);

};

class TransitionCostComputation_MultiFactor : public TransitionCostComputation {
private:

	uint32_t form_lm_order;
	uint32_t lemma_lm_order;
	uint32_t tag_lm_order;

	double form_weight;
	double lemma_weight;
	double tag_weight;

public:

	TransitionCostComputation_MultiFactor(uint32_t _form_lm_order, uint32_t _lemma_lm_order, uint32_t _tag_lm_order,
			double _form_weight, double _lemma_weight, double _tag_weight):
		form_lm_order(_form_lm_order), lemma_lm_order(_lemma_lm_order), tag_lm_order(_tag_lm_order),
		form_weight(_form_weight), lemma_weight(_lemma_weight), tag_weight(_tag_weight)
	{}

	virtual double ComputeTransitionCost(ViterbiStateP state, StagePosibilityP next);

	virtual double ComputeTransitionCostSPSequence(vector<StagePosibilityP> &sp_vec, uint32_t start_index, uint32_t end_index);

};

class TransitionCostComputation_MorphoAnal : public TransitionCostComputation {
private:
	double lemma_weight;
	double tag_weight;
public:

	TransitionCostComputation_MorphoAnal(double _lemma_weight, double _tag_weight):
		lemma_weight(_lemma_weight), tag_weight(_tag_weight)
	{}

	virtual double ComputeTransitionCost(ViterbiStateP state, StagePosibilityP next);

	virtual double ComputeTransitionCostSPSequence(vector<StagePosibilityP> &sp_vec, uint32_t start_index, uint32_t end_index);

};

class TransitionCostComputation_Letter : public TransitionCostComputation {

	bool is_name;

public:

	TransitionCostComputation_Letter(bool _is_name): is_name(_is_name) {}

	virtual double ComputeTransitionCost(ViterbiStateP state, StagePosibilityP next);

	virtual double ComputeTransitionCostSPSequence(vector<StagePosibilityP> &sp_vec, uint32_t start_index, uint32_t end_index);

};

}

#endif /* TRANSITIONCOSTCOMPUTATION_HPP_ */
