/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "ViterbiState.hpp"
#include "StagePosibility.hpp"

namespace ngramchecker {

		bool ViterbiState::CheckIt()
		{

			for (uint32_t i = 0; i < history.size(); i++)
			{
				if (history[i]->type != type)
					return false;
			}

			return true;
		}

		StagePosibilityP ViterbiState::GetYoungestHistory()
		{
			return history[history.size() - 1];
		}

		size_t ViterbiState::UniqueIdentifier()
		{
			size_t seed = 0;

			boost::hash_combine(seed, history.size());

			for (uint32_t i = 0; i < history.size(); i++)
			{
				boost::hash_combine(seed, history[i]->UniqueIdentifier());
			}

			return seed;

		}

		bool ViterbiState::Equals(ViterbiState &state)
		{
			return this->UniqueIdentifier() == state.UniqueIdentifier();
		}

		ViterbiState::ViterbiState(vector<StagePosibilityP> _history):
			history(_history), distance(0), ancestor(ViterbiStateP())
		{
			type = _history[0]->type;
			assert(CheckIt());
		}

		ViterbiState::ViterbiState(ViterbiStateP prev_state, StagePosibilityP next_sp, double _distance)
		{
			type = next_sp->type;

			for (size_t i = 1; i < prev_state->history.size(); i++)
			{
				history.push_back(prev_state->history[i]);
			}

			history.push_back(next_sp);

			distance = _distance;

			ancestor = prev_state;
			assert(CheckIt());
		}

		string ViterbiState::ToString()
		{
			stringstream strs;
			strs << "(Old -> Young): ";

			for (uint32_t i = 0; i < history.size(); i++)
			{
				if (i > 0) strs << ", ";
				strs << history[i]->ToString();
			}

			return strs.str();
		}
}