/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef VITERBISTATE_HPP_
#define VITERBISTATE_HPP_

#include "StdAfx.h"

namespace ngramchecker {

	class ViterbiState;
	SP_DEF(ViterbiState);

	class StagePosibility;
	SP_DEF(StagePosibility);

	class ViterbiState
	{
	public:
		StagePosibility_type type;

		vector<StagePosibilityP> history;

		double distance;
		ViterbiStateP ancestor;

	private:

		bool CheckIt();

	public:

		StagePosibilityP GetYoungestHistory();

		size_t UniqueIdentifier();

		bool Equals(ViterbiState &state);

		ViterbiState(vector<StagePosibilityP> _history);

		ViterbiState(ViterbiStateP prev_state, StagePosibilityP next_sp, double _distance);

		string ToString();
	};




	struct ViterbiStateP_compare_distance : std::less<ViterbiStateP>
	{
		bool operator()(const ViterbiStateP& s1, const ViterbiStateP& s2)
		{
			return s1->distance < s2->distance;
		}
	};

	struct ViterbiStateP_iequal_to
		: std::binary_function<ViterbiStateP, ViterbiStateP, bool>
	{
		bool operator()(ViterbiStateP const& x,
			ViterbiStateP const& y) const
		{
			return x->Equals(*y);
		}
	};


	struct ViterbiStateP_ihash: std::unary_function<ViterbiStateP, std::size_t>
	{
		std::size_t operator()(ViterbiStateP const& x) const
		{
			return x->UniqueIdentifier();
		}
	};



}

#endif /* VITERBISTATE_HPP_ */
