/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#include "ZipLM.hpp"
#include "MyPackedArray.hpp"
#include "MyMappedDoubleArray.hpp"
#include "CompIncreasingArray.hpp"
#include "NGram.hpp"

namespace ngramchecker {

		LM_tuple::LM_tuple(uint32_t _id, int _higher_ngram_offset_start, int _higher_ngram_offset_end, double _prob, double _bow)
		{
			word_id = _id;
			higher_ngram_offset_start = _higher_ngram_offset_start;
			higher_ngram_offset_end = _higher_ngram_offset_end;
			prob = _prob;
			bow = _bow;
		}

		int ZipLM::search_for_id(uint32_t _order, uint32_t word_id, int start_offset, int end_offset)
		{
			if (end_offset < start_offset)
			{
				//cerr << "end_offset = " << end_offset << ", start_offset = " << start_offset << endl;
				return -1;
			}

			if (_order == 0)
			{
				assert( (start_offset == 0) && (end_offset == 0) );
				if (word_id > max_unigram_id)
					return -1;
				else
					return word_id;
			}

			/*if (_order == 1)
			{
				uint32_t _offset = offsets[0]->GetValueAt(id);

				assert( (start_offset == 0) && (end_offset == 0) );

				if (id < offsets[0]->GetSize() - 1)
				{
					if (offsets[0]->GetValueAt(id + 1) == _offset)
					{
						return -1;
					}
				}
				else
				{
					if (_offset >= offsets[1]->GetSize())
					{
						return -1;
					}
				}

				return _offset;
			}*/

			uint32_t index;

			MyPackedArrayP id_arr = ids[_order];

			while (end_offset - start_offset > 1)
			{
				index = (start_offset + end_offset) / 2;
				if (id_arr->GetValueAt(index) < word_id)
				{
					start_offset = index;
				}
				else
				{
					end_offset = index;
				}
			}

			if (id_arr->GetValueAt(start_offset) == word_id)
				return start_offset;
			else if (id_arr->GetValueAt(end_offset) == word_id)
				return end_offset;
			else
				return -1;
		}

		LM_tupleP ZipLM::get_lm_tuple(uint32_t _order, uint32_t _offset)
		{

			double prob = probs[_order]->GetValueAt(_offset);
			double bow;

			if (_order < lm_order - 1)
				bow = bows[_order]->GetValueAt(_offset);
			else
				bow = 0;

			uint32_t word_id;
			if (_order > 0)
				word_id = ids[_order]->GetValueAt(_offset);
			else
				word_id = _offset;

			if (_order < lm_order - 1)
			{
				CompIA_First_Last_IndexPair offset_pair = offsets[_order]->GetFirstLastIndexPair(_offset);

				if (offset_pair != NULL)
				{
					return LM_tupleP(new LM_tuple(word_id, offset_pair->first, offset_pair->second, prob, bow));
				}
				else
					return LM_tupleP(new LM_tuple(word_id, 1, 0, prob, bow));
			}
			else
			{
				return LM_tupleP(new LM_tuple(word_id, 1, 0, prob, bow));
			}
		}

		void ZipLM::GetAllNGramsPerUnigramID_impl(uint32_t order, int start_index, int end_index, vector<uint32_t> lo_ids, vector<NGramP> &ret)
		{
			//cerr << "order = " << order << ", start_index = " << start_index << ", end_index = " << end_index << endl;
			//cerr << ret.size() << endl;
			for (int i = start_index; i <= end_index; i++)
			{
				//if (order == 0)
				//	cerr << "i at 0 = " << i << ", end_index == " << end_index << endl;
				//cerr << "i = " << i << endl;
				LM_tupleP tup = get_lm_tuple(order, i);

				uint32_t *ngram_ids = new uint32_t[order + 1];

				assert(lo_ids.size() == order);

				for (uint32_t j = 0; j < lo_ids.size(); j++)
				{
					ngram_ids[j] = lo_ids[j];
				}

				ngram_ids[order] = tup->word_id;

				NGramP ngram = NGram::createNGramPointer(order + 1, ngram_ids, tup->prob, tup->bow);
				ret.push_back(ngram);

				//cerr << ngram->ToString() << endl;

				/*if (i == 0)
				{
					cerr << "tup->higher_ngram_offset_start = " << tup->higher_ngram_offset_start <<
							"tup->higher_ngram_offset_end = " << tup->higher_ngram_offset_end << endl;
				}*/

				if (order < lm_order - 1)
				{
					lo_ids.push_back(tup->word_id);
					GetAllNGramsPerUnigramID_impl(order + 1, tup->higher_ngram_offset_start, tup->higher_ngram_offset_end, lo_ids, ret);
					lo_ids.pop_back();
				}
			}
		}

		vector<NGramP> ZipLM::GetAllNGramsPerUnigramID(uint32_t unigramID)
		{
			vector<NGramP> ret;

			if (unigramID >= probs[0]->GetSize())
				return ret;

			GetAllNGramsPerUnigramID_impl(0, unigramID, unigramID, vector<uint32_t>(), ret);

			return ret;
		}

		string ZipLM::GetFilename()
		{
			return filename;
		}

		double ZipLM::GetWordNotInLMCost()
		{
			return not_in_lm_cost;
		}


		NGramP ZipLM::GetNGramForNGramKey(NGramP ngram)
		{
			if (ngram->order > lm_order)
			{
				cerr << "ngram->order = " << ngram->order << ", lm_order = " << lm_order << endl;
			}
			assert(ngram->order <= lm_order);

			uint32_t input_order = ngram->order;

			int start_offset = 0;
			int end_offset = 0;

			vector<uint32_t> ids;

			LM_tupleP lm_tup = LM_tupleP();

			for (uint32_t order = 0; order < input_order; order++)
			{
				uint32_t tok_id = ngram->GetWordID_New(order);
				int offset = search_for_id(order, tok_id, start_offset, end_offset);

				if (offset == -1)
					break;

				lm_tup = get_lm_tuple(order, offset);

				ids.push_back(tok_id);

				start_offset = lm_tup->higher_ngram_offset_start;
				end_offset = lm_tup->higher_ngram_offset_end;
			}

			if (lm_tup == NULL)
				return NGramP();
			else
			{
				uint32_t* c_ids = new uint32_t[ids.size()];

				for (uint32_t i = 0; i < ids.size(); i++)
					c_ids[i] = ids[i];

				return NGram::createNGramPointer(ids.size(), c_ids, lm_tup->prob, lm_tup->bow);
			}

		}

		ZipLM::ZipLM(uint32_t _order, double _not_in_lm_cost, vector<vector<double> > &_probs, vector<vector<double> > &_bows,
				vector<vector<uint32_t> > &_ids, vector<vector<uint32_t> > &_offsets)
		{
			lm_order = _order;
			not_in_lm_cost = _not_in_lm_cost;

			max_unigram_id = _probs[0].size() - 1;
			cerr << "LM order: " << _order << endl;

			ids.push_back(MyPackedArrayP());

			for (uint32_t i = 0; i < _order; i++)
			{
				uint32_t bits_per_prob;

				if (i == 0)
					bits_per_prob = bits_per_unigram_prob;
				else if (i == 1)
					bits_per_prob = bits_per_bigram_prob;
				else
					bits_per_prob = bits_per_higher_order_prob;

				//cerr << "creating probs order " << i << endl;
				probs.push_back(MyMappedDoubleArrayP(new MyMappedDoubleArray(_probs[i], bits_per_prob)));
			}

			for (uint32_t i = 0; i < _order - 1; i++)
			{
				uint32_t bits_per_bow;

				if (i == 0)
					bits_per_bow = bits_per_unigram_bow;
				else if (i == 1)
					bits_per_bow = bits_per_bigram_bow;
				else
					bits_per_bow = bits_per_higher_order_bow;

				//cerr << "creating bows order " << i << endl;
				bows.push_back(MyMappedDoubleArrayP(new MyMappedDoubleArray(_bows[i], bits_per_bow)));
			}

			for (uint32_t i = 1; i < _order; i++)
			{
				//cerr << "creating ids order " << i << endl;
				ids.push_back(MyPackedArrayP(new MyPackedArray(_ids[i])));
			}

			for (uint32_t i = 0; i < _order - 1; i++)
			{

				offsets.push_back(CompIncreasingArrayP(new CompIncreasingArray(_offsets[i], ids[i + 1]->GetSize() - 1)));
			}
		}

		ZipLM::ZipLM(string bin_file)
		{
			filename = bin_file;
			ifstream ifs;
			ifs.open(bin_file.c_str(), ios::in | ios::binary);

			/*if (! ifs.is_open())
			{
				cerr << "Opening: " << bin_file << endl;
			}*/
			assert(ifs.is_open());

			ifs.read((char*)&lm_order, sizeof(uint32_t));
			ifs.read((char*)&not_in_lm_cost, sizeof(double));

			ids.push_back(MyPackedArrayP());

			for (uint32_t i = 0; i < lm_order; i++)
			{
				probs.push_back(MyMappedDoubleArrayP(new MyMappedDoubleArray(ifs)));
			}

			max_unigram_id = probs[0]->GetSize() - 1;

			for (uint32_t i = 0; i < lm_order - 1; i++)
			{
				bows.push_back(MyMappedDoubleArrayP(new MyMappedDoubleArray(ifs)));
			}

			for (uint32_t i = 1; i < lm_order; i++)
			{
				ids.push_back(MyPackedArrayP(new MyPackedArray(ifs)));
			}

			for (uint32_t i = 0; i < lm_order - 1; i++)
			{
				offsets.push_back(CompIncreasingArrayP(new CompIncreasingArray(ifs)));
			}

			ifs.close();

		}

		void ZipLM::SaveInBinaryForm(string out_file)
		{
			cerr << "Saving in binary form..." << endl;
			ofstream ofs;
			ofs.open(out_file.c_str(), ios::out | ios::binary);
			assert(ofs.is_open());

			ofs.write((char*)&lm_order, sizeof(uint32_t));
			ofs.write((char*)&not_in_lm_cost, sizeof(double));

			for (uint32_t i = 0; i < lm_order; i++)
			{
				probs[i]->WriteToStream(ofs);
			}

			for (uint32_t i = 0; i < lm_order - 1; i++)
			{
				bows[i]->WriteToStream(ofs);
			}

			for (uint32_t i = 1; i < lm_order; i++)
			{
				ids[i]->WriteToStream(ofs);
			}

			for (uint32_t i = 0; i < lm_order - 1; i++)
			{
				offsets[i]->WriteToStream(ofs);
			}


			ofs.close();
		}

		ZipLM ZipLM::createFromTextFile(string text_file)
		{
			cerr << "Creating from text file..." << endl;
			ifstream ifs;
			ifs.open(text_file.c_str());
			assert(ifs.is_open());

			string s;

			std::getline(ifs, s);

			uint32_t lm_order = boost::lexical_cast<uint32_t>(s);

			vector<vector<double> > probs;
			vector<vector<double> > bows;
			vector<vector<uint32_t> > ids;

			vector<vector<uint32_t> > offsets;

			ids.push_back(vector<uint32_t>());

			for (uint32_t i = 0; i < lm_order; i++)
			{
				probs.push_back(vector<double>());

				if (i > 0)
					ids.push_back(vector<uint32_t>());

				if (i < lm_order - 1)
				{
					offsets.push_back(vector<uint32_t>());
					bows.push_back(vector<double>());
				}
			}


			std::getline(ifs, s);

			double not_in_lm_cost = boost::lexical_cast<double>(s);
			uint32_t max_id = 0;

			int last_unigram_id = -1;
			int unigram_id;

			while (std::getline(ifs, s))
			{
				vector<string> toks;
				boost::split(toks, s, boost::is_any_of(" "));

				uint32_t ngram_order = toks.size() - 3;

				if (ngram_order < lm_order - 1)
				{
					offsets[ngram_order].push_back(ids[ngram_order + 1].size());
					bows[ngram_order].push_back(boost::lexical_cast<double>(toks[toks.size() - 1]));
				}

				probs[ngram_order].push_back(boost::lexical_cast<double>(toks[toks.size() - 2]));

				uint32_t new_id = boost::lexical_cast<uint32_t>(toks[toks.size() - 3]);

				if (new_id > max_id)
					max_id = new_id;

				if (ngram_order > 0)
				{
					ids[ngram_order].push_back(new_id);
				}
				else
				{
					unigram_id = new_id;

					if (unigram_id != last_unigram_id + 1)
					{
						cerr << "unigram_id = " << unigram_id << ", last_unigram_id = " << last_unigram_id << endl;

					}
					assert(unigram_id == last_unigram_id + 1);
					last_unigram_id = unigram_id;
				}
			}

			/*uint32_t index = 0;
			uint32_t next_unigram_id = ids[1][index];

			for (uint32_t i = 0; i <= max_id; i++)
			{
				if (i > next_unigram_id)
				{
					index++;
					next_unigram_id = ids[1][index];

					assert(i <= next_unigram_id);
				}

				offsets[0].push_back(index);
			}

			ids[1] = vector<uint32_t>();*/

			return ZipLM(lm_order, not_in_lm_cost, probs, bows, ids, offsets);
		}

		uint32_t ZipLM::MaxUnigramID()
		{
			return probs[0]->GetSize() - 1;
		}

		void ZipLM::TestIt(string textFile)
		{
			ifstream istr;
			istr.open(textFile.c_str());
			istr.is_open();

			string s;

			std::getline(istr, s);
			std::getline(istr, s);

			uint32_t counter = 0;
			uint32_t counter_big_diff = 0;

			while (std::getline(istr, s))
			{
				counter++;
				vector<string> parts;

				boost::split(parts, s, boost::is_any_of(" "));

				uint32_t order = parts.size() - 2;

				uint32_t *key_ids = new uint32_t[order];

				for (uint32_t i = 0; i < order; i++)
				{
					key_ids[i] = boost::lexical_cast<uint32_t>(parts[i]);
				}

				double text_prob = boost::lexical_cast<double>(parts[parts.size() - 2]);
				double text_bow = boost::lexical_cast<double>(parts[parts.size() - 1]);

				NGramP ngram_key = NGram::createNGramSearchKey(order, key_ids);

				NGramP ngram = GetNGramForNGramKey(ngram_key);

				if (ngram == NULL)
				{
					cerr << "problem: ngram == NULL: " << ngram_key->ToString() << endl;
				}

				assert(ngram != NULL);

				assert(ngram->order == ngram_key->order);

				if(fabs(text_prob - ngram->prob) > 0.5 || fabs(text_bow - ngram->backoff) > 0.5)
				{
					cerr << "text prob: " << text_prob << ", ZipLM prob: " << ngram->prob;
					cerr << "text bow: " << text_bow << ", ZipLM bow: " << ngram->backoff;
					cerr << endl;
					counter_big_diff++;
				}
			}

			cerr << "ngrams: " << counter << endl;
			cerr << "num of big diffs: " << counter_big_diff << endl;
			cerr << "OK!" << endl;
		}

	const uint32_t ZipLM::bits_per_unigram_prob = 8;
	const uint32_t ZipLM::bits_per_unigram_bow = 8;
	const uint32_t ZipLM::bits_per_bigram_prob = 8;
	const uint32_t ZipLM::bits_per_bigram_bow = 8;
	const uint32_t ZipLM::bits_per_higher_order_prob = 8;
	const uint32_t ZipLM::bits_per_higher_order_bow = 8;


}
