/*
 
 Copyright (C) 2010 Michal Richter (michalisek@gmail.com)
 
 This file is part of michalisekSpell.
 
 MichalisekSpell is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
 along with michalisekSpell; see the file COPYING.  If not, see
 <http://www.gnu.org/licenses/>.
 
 */

#ifndef ZIPLM_HPP_
#define ZIPLM_HPP_

#include "StdAfx.h"

namespace ngramchecker {

	class MyPackedArray;
	SP_DEF(MyPackedArray);

	class MyMappedDoubleArray;
	SP_DEF(MyMappedDoubleArray);

	class CompIncreasingArray;
	SP_DEF(CompIncreasingArray);

	class NGram;
	SP_DEF(NGram);

	struct LM_tuple {
		uint32_t word_id;
		int higher_ngram_offset_start;
		int higher_ngram_offset_end;

		double prob;
		double bow;

		LM_tuple(uint32_t _id, int _higher_ngram_offset_start, int _higher_ngram_offset_end, double _prob, double _bow);

	};

	SP_DEF(LM_tuple);

	class ZipLM {
	private:

		static const uint32_t bits_per_unigram_prob;
		static const uint32_t bits_per_unigram_bow;
		static const uint32_t bits_per_bigram_prob;
		static const uint32_t bits_per_bigram_bow;
		static const uint32_t bits_per_higher_order_prob;
		static const uint32_t bits_per_higher_order_bow;


		vector<MyMappedDoubleArrayP> probs;
		vector<MyMappedDoubleArrayP> bows;
		vector<MyPackedArrayP> ids;

		vector<CompIncreasingArrayP> offsets;

		vector<uint32_t> bits_per_probs;
		vector<uint32_t> bits_per_bows;

		uint32_t lm_order;
		double not_in_lm_cost;
		string filename;

		uint32_t max_unigram_id;

		int search_for_id(uint32_t _order, uint32_t word_id, int start_offset, int end_offset);

		LM_tupleP get_lm_tuple(uint32_t _order, uint32_t _offset);

		void GetAllNGramsPerUnigramID_impl(uint32_t order, int start_index, int end_index, vector<uint32_t> lo_ids, vector<NGramP> &ret);

	public:

		vector<NGramP> GetAllNGramsPerUnigramID(uint32_t unigramID);

		string GetFilename();

		double GetWordNotInLMCost();

		NGramP GetNGramForNGramKey(NGramP gram);

		ZipLM(uint32_t _order, double _not_in_lm_cost, vector<vector<double> > &_probs, vector<vector<double> > &_bows,
				vector<vector<uint32_t> > &_ids, vector<vector<uint32_t> > &_offsets);

		ZipLM(string bin_file);

		void SaveInBinaryForm(string out_file);

		static ZipLM createFromTextFile(string text_file);

		uint32_t MaxUnigramID();

		void TestIt(string textFile);
	};

	SP_DEF(ZipLM);
}

#endif /* ZIPLM_HPP_ */
