/*

Copyright (C) 2010 Michal Richter (michalisek@gmail.com)

This file is part of michalisekSpell.

MichalisekSpell is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with michalisekSpell; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "StdAfx.h"
#include "createLexiconAutomaton.hpp"
#include "GlobalVars.hpp"
#include "Lexicon.hpp"
#include "StringMapper.hpp"

namespace ngramchecker {

		void LexiconCreator::initFormIdMap()
		{


			string fileName = MyConstants::project_home + "/data/morphology/text/form.out";

			ifstream istr;
			istr.open(fileName.c_str());

			assert(istr.is_open());

			string s;

			while (std::getline(istr, s))
			{
				if (s == "`")
				{
					cerr << "initFormIdMap: ` found, started mapping of real words..." << endl;
					break;
				}
			}

			uint32_t counter = 0;
			while(std::getline(istr, s))
			{

				form_string_id_map[s] = counter;
				if (counter % 1000 == 0)
					cerr << "form_string_id_map[" << s << "] = " << counter << endl;
				counter++;

			}

			istr.close();
		}

		void LexiconCreator::createLexAutNew()
		{
			string s;

			ofstream ostr;
			ifstream istr;
			istr.open((MyConstants::project_home + "/data/morphology/text/form.out").c_str());

			assert(istr.is_open());


			vector< vector<pair<uint32_t, uint32_t> > > states;
			uint32_t nextStateID = 1;

			states.push_back(vector<pair<uint32_t, uint32_t> >());

			while (std::getline(istr, s))
			{
				if (s == "`")
				{
					cerr << "` found, start to collect real words";
					break;
				}
			}


			uint32_t counter = 0;

			while (std::getline(istr, s))
			{
				if (s == ".")
					break;

				counter++;

				if (counter % 20000 == 0)
				{
					cerr << counter << endl;
				}

				uint32_t currState = 0;

				uint32_t char_num;
				for (uint32_t i = 0; i < s.length(); i++)
				{
					char_num = GlobalVars::stringMapper->MapChar(s[i]);

					vector<pair<uint32_t, uint32_t> > currStateVector = states[currState];

					int index = -1;

					for (uint32_t g = 0; g < currStateVector.size(); g++)
					{
						if (currStateVector[g].first == char_num)
						{
							index = g;
							break;
						}
					}

					if (index == -1)
					{
						states[currState].push_back(pair<uint32_t, uint32_t>(char_num, nextStateID));
						assert(nextStateID == states.size());
						states.push_back(vector<pair<uint32_t, uint32_t> >());
						currState = nextStateID;
						nextStateID++;
					}
					else
					{
						currState = currStateVector[index].second;
					}
				}
			}

			for (uint32_t i = 0; i < states.size(); i++)
			{
				std::sort(states[i].begin(), states[i].end());
			}

			cerr << "first phase finished, random paths:\n";

			/*for (int i = 0; i < 100; i++)
			{
				vector<pair<uint32_t, uint32_t> > arcs = states[0];
				uint32_t stateID = 0;
				string slovo = "";
				while (arcs.size() > 0)
				{
					//cout << "ble";
					uint32_t edge_ind = (uint32_t)randomR(0, arcs.size());
					pair<uint32_t, uint32_t> arc = arcs[edge_ind];
					slovo += stringMapper.InvMapChar(arc.first);
					stateID = arc.second;
					arcs = states[stateID];

				}
				cerr << "slovo: " << slovo << endl;

			}

			assert(1 == 0);*/
			initFormIdMap();

			vector<uint32_t> acumID;
			vector<uint32_t > acumArcPointer;
			acumID.push_back(0);
			acumArcPointer.push_back(0);
			map<uint32_t, int> automat_id_remap;
			uint32_t sID;
			uint32_t currArcPointer;
			uint32_t lastNewID = 0;
			string prefix = "";

			while (acumArcPointer.size() > 0)
			{
				sID = acumID[acumID.size() - 1];
				if (automat_id_remap.find(sID) == automat_id_remap.end())
				{

					if (form_string_id_map.find(prefix) == form_string_id_map.end())
					{
						automat_id_remap[sID] = lastNewID;
						lastNewID++;
					}
					else
					{

						automat_id_remap[sID] = - (int)(form_string_id_map[prefix]) - 1;
					}
				}

				currArcPointer = (acumArcPointer[acumArcPointer.size() - 1]);

				if (currArcPointer < states[sID].size())
				{
					prefix = prefix + GlobalVars::stringMapper->InvMapChar(states[sID][currArcPointer].first);
					acumID.push_back(states[sID][currArcPointer].second);
					acumArcPointer[acumArcPointer.size() - 1]++;
					acumArcPointer.push_back(0);
				}
				else
				{
					prefix = prefix.substr(0, prefix.length() - 1);
					acumArcPointer.pop_back();
					acumID.pop_back();
				}

			}

			uint32_t n_inner_states = lastNewID;

			map<uint32_t, int> automat_id_remap2;

			uint32_t maxNewID = 0;
			for (map<uint32_t, int>::iterator it = automat_id_remap.begin(); it != automat_id_remap.end(); it++)
			{
				if (it->second < 0)
					automat_id_remap2[it->first] = (- it->second) + lastNewID - 1;
				else
					automat_id_remap2[it->first] = it->second;

				if (automat_id_remap2[it->first] > (int)maxNewID)
					maxNewID = automat_id_remap2[it->first];

			}


			map<uint32_t, map<uint32_t, uint32_t> > states_new;

			uint32_t remap_id_1;
			uint32_t remap_id_2;

			for (uint32_t i = 0; i < states.size(); i++)
			{
				remap_id_1 = automat_id_remap2[i];

				states_new[remap_id_1] = map<uint32_t, uint32_t>();
				vector<pair<uint32_t, uint32_t> > pom_vec = states[i];

				for (uint32_t j = 0; j < states[i].size(); j++)
				{
					remap_id_2 = automat_id_remap2[states[i][j].second];
					states_new[remap_id_1][states[i][j].first] = remap_id_2;
				}
			}

			Lexicon lex = Lexicon(states_new, n_inner_states);

			cerr << "Instance of Lexicon class created\n";

			lex.WriteToBinFile(MyConstants::project_home + "/data/morphology/bin/lexicon.bin");

			lex.TestIt();
		}

}
