/*

Copyright (C) 2010 Michal Richter (michalisek@gmail.com)

This file is part of michalisekSpell.

MichalisekSpell is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

MichalisekSpell is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with MichalisekSpell; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/


#include "StdAfx.h"
#include "GlobalVars.hpp"
#include "StringMapper.hpp"
#include "TextCheckingServer.hpp"
#include "Splitter.hpp"
#include "ErrorModelBasic_ble.hpp"

	void set_project_home(string ngramchecker_filename)
	{
		int index = ngramchecker_filename.find("michalisekSpell_proj/Debug/michalisekSpell");

		if (index == -1)
		{
			index = ngramchecker_filename.find("VC2010");//\\korektor\\debug\\korektor.exe");
			if (index == -1)
			{
				index = ngramchecker_filename.length() - 15;
			}
		}

		MyConstants::project_home = ngramchecker_filename.substr(0, index);

		cerr << "project_home = " << MyConstants::project_home << endl;

		MyConstants::filename_lexicon_bin = MyConstants::project_home + "data/morphology/bin/lexicon.bin";
		MyConstants::filename_morphology_bin = MyConstants::project_home + "data/morphology/bin/morphology.bin";

		MyConstants::filename_form_zip_lm_bin = MyConstants::project_home + "data/lm/bin/form-zip.bin";
		MyConstants::filename_lemma_zip_lm_bin = MyConstants::project_home + "data/lm/bin/lemma-zip.bin";
		MyConstants::filename_tag_zip_lm_bin = MyConstants::project_home + "data/lm/bin/tag-zip.bin";
		MyConstants::filename_letter_universal_zip_lm_bin = MyConstants::project_home + "data/lm/bin/letter-zip.bin";
		MyConstants::filename_letter_names_zip_lm_bin = MyConstants::project_home + "data/lm/bin/letter-names-zip.bin";
		MyConstants::filename_letter_common_zip_lm_bin = MyConstants::project_home + "data/lm/bin/letter-common-zip.bin";

		MyConstants::filename_test_sentence = MyConstants::project_home + "data/sentence.test";
		MyConstants::filename_chars_list = MyConstants::project_home + "data/chars.list";
		MyConstants::filename_utf8_list = MyConstants::project_home + "data/diakritik.utf8";
		MyConstants::filename_iso_list = MyConstants::project_home + "data/diakritik.iso";
		MyConstants::filename_error_model_diacritics = MyConstants::project_home + "data/errorModel-diacritics";
		MyConstants::filename_error_model_typo_manual = MyConstants::project_home + "data/errorModel-typo";
		MyConstants::filename_error_model_type_auto = MyConstants::project_home + "data/errorModel-typo_new";
		MyConstants::filename_lemma_signs = MyConstants::project_home + "data/morphology/bin/lemma_signs.bin";
		MyConstants::filename_keyboard_layout = MyConstants::project_home + "/data/keyboard_layout.new";
		MyConstants::filename_form_map = MyConstants::project_home + "/data/morphology/text/form.out";
		MyConstants::filename_lemma_map = MyConstants::project_home + "/data/morphology/text/lemma.out";
		MyConstants::filename_tag_map = MyConstants::project_home + "/data/morphology/text/tag.out";
		MyConstants::filename_emmisions_bin = MyConstants::project_home + "/data/lm/bin/emmisions.bin";
		MyConstants::filename_lc_uc_letters = MyConstants::project_home + "/data/chars_lc_uc.list";
		MyConstants::directory_evaluation_correction = MyConstants::project_home + "/evaluation/correction/";
		MyConstants::directory_evaluation_diacritic = MyConstants::project_home + "/evaluation/diacritic/";
		MyConstants::directory_evaluation_tagger = MyConstants::project_home + "/evaluation/tagger/";

		MyConstants::filename_evaluation_correction_heldout_prefix = MyConstants::project_home + "/evaluation/correction/heldout";
		MyConstants::filename_evaluation_correction_test_prefix = MyConstants::project_home + "/evaluation/correction/test";


	}

	int main(int argc, char** argv) {


		string ngramchecker_filename = argv[0];
		set_project_home(ngramchecker_filename);

		string encoding = "utf8";
		string task = "spellchecking";

		string error_model_file = "";

		float error_weight = 1;

			vector<string> args;

			for (size_t i = 1; i < (size_t)argc; i++)
			{
			    //cout << argv[i] << endl;
				args.push_back(argv[i]);
			}

			size_t i = 0;
			while (i < args.size())
			{
				if (args[i] == "-fw")
				{
					i++;
					MyConstants::form_weight = boost::lexical_cast<float>(args[i]);
				}
				else if (args[i] == "-lw")
				{
					i++;
					MyConstants::lemma_weight = boost::lexical_cast<float>(args[i]);
				}
				else if (args[i] == "-tw")
				{
					i++;
					MyConstants::tag_weight = boost::lexical_cast<float>(args[i]);
				}
				else if (args[i] == "-ew")
				{
					i++;
					error_weight = boost::lexical_cast<float>(args[i]);
				}
				else if (args[i] == "-fo")
				{
					i++;
					MyConstants::form_order = boost::lexical_cast<uint32_t>(args[i]);
				}
				else if (args[i] == "-lo")
				{
					i++;
					MyConstants::lemma_order = boost::lexical_cast<uint32_t>(args[i]);
				}
				else if (args[i] == "-to")
				{
					i++;
					MyConstants::tag_order = boost::lexical_cast<uint32_t>(args[i]);
				}
				else if (args[i] == "-fm")
				{
					i++;
					MyConstants::filename_form_zip_lm_bin = args[i];
				}
				else if (args[i] == "-lm")
				{
					i++;
					MyConstants::filename_lemma_zip_lm_bin = args[i];
				}
				else if (args[i] == "-tm")
				{
					i++;
					MyConstants::filename_tag_zip_lm_bin = args[i];
				}
				else if (args[i] == "-lo")
				{
					i++;
					MyConstants::letter_decoder_order = boost::lexical_cast<uint32_t>(args[i]);
				}
				else if (args[i] == "-lm")
				{
					i++;
					MyConstants::filename_letter_universal_zip_lm_bin = args[i];
				}
				else if (args[i] == "-error-model")
				{
					i++;
					error_model_file = args[i];
				}
				else if (args[i] == "-letter")
					MyConstants::use_letter_lm = true;
				else if (args[i] == "-dc-ignore-case")
					MyConstants::diacritic_completion_ignore_case = true;
				else if (args[i] == "-spellchecking-eval")
					task = "spellchecking-eval";
				else if (args[i] == "-spellchecking-eval+diacritics")
				{
					task = "spellchecking-eval";
					MyConstants::spellchecking_diac_zero_dist = true;
				}
				else if (args[i] == "-spellchecking")
				    task = "spellchecking";
				else if (args[i] == "-spellchecking+diacritics")
				{
					task = "spellchecking";
					MyConstants::spellchecking_diac_zero_dist = true;
				}
				else if (args[i] == "-tokenize")
					task = "tokenize";
				else if (args[i] == "-autocorrect")
				    task = "autocorrect";
				else if (args[i] == "-autocorrect+diacritics")
				{
					task = "autocorrect";
					MyConstants::spellchecking_diac_zero_dist = true;
				}
				else if (args[i] == "-diacritics")
				    task = "diacritics";
				else if (args[i] == "-remove-diacritics")
				    task = "remove-diacritics";
				else if (args[i] == "-abbreviate")
					task = "abbreviate";
				else if (args[i] == "-abbreviation-complete")
					task = "abbreviation-complete";
				else if (args[i] == "-utf8")
				    encoding = "utf8";
				else if (args[i] == "-iso")
				    encoding = "iso";
				else if (args[i] == "-help")
				{
				
				    std::cerr << "Uses stdin and stdout for input and output!\n"
				              << "List of the valid parameters:\n\n"
				              << "-spellchecking : checks the input string for the spelling errors, the spelling errors will be marked with the XML-like tags (default option)\n"
				              << "-autocorrect : checks the input string for the spelling errors,, the spelling errors will be corrected automatically\n"
				              << "-diacritics : completes the diacritics\n"
				              << "-utf8 : sets the input/output encoding to UTF-8 (default)\n"
				              << "-iso : sets the input/output encoding to ISO-8859-2\n"
				              << "-help : displays this information\n";
				}
				else
				{
				    cerr << "Invalid parameter: " << args[i] << std::endl << "Run the program with \"-help\" parameter to see the list of all valid parameters!\n";
				    exit(1);
				}
				
				i++;
			}

			MyConstants::form_weight /= error_weight;
			MyConstants::lemma_weight /= error_weight;
			MyConstants::tag_weight /= error_weight;

			GlobalVars::InitGlobalVars();
			
			if (error_model_file != "")
				GlobalVars::errorModelTypoP = ErrorModelBasic::fromBinaryFile(error_model_file);

			//cerr << "args[0]: " << args[0] << endl;

			if (task == "diacritics")
			{
				string str;
				while (std::getline(cin, str))
				{
				    if (encoding == "utf8")
				        str = GlobalVars::stringMapper->ConvertToISO(str);
				    
					string output = GlobalVars::textCheckingServer->AddDiacritics(str);

                    if (encoding == "utf8")
                        output =  GlobalVars::stringMapper->ConvertBackFromISO(output);

					cout << output << endl;
				}
			}
			else if (task == "remove-diacritics")
			{
				string str;
				while (std::getline(cin, str))
				{
   				    if (encoding == "utf8")
				        str = GlobalVars::stringMapper->ConvertToISO(str);

					string output = GlobalVars::textCheckingServer->RemoveDiacritics(str);

                    if (encoding == "utf8")
                        output =  GlobalVars::stringMapper->ConvertBackFromISO(output);

					cout << output << endl;
				}			
			}
			else if (task == "autocorrect")
			{
				string str;
				while (std::getline(cin, str))
				{
    				if (encoding == "utf8")
				        str = GlobalVars::stringMapper->ConvertToISO(str);

					string output = GlobalVars::textCheckingServer->AutoCorrect(str);

                    if (encoding == "utf8")
                        output =  GlobalVars::stringMapper->ConvertBackFromISO(output);

					cout << output << endl;
				}
			}
			else if (task == "abbreviate")
			{
				string str;
				while (std::getline(cin, str))
				{
    				if (encoding == "utf8")
				        str = GlobalVars::stringMapper->ConvertToISO(str);

					string output = GlobalVars::textCheckingServer->AbbreviateText(str);

                    if (encoding == "utf8")
                        output =  GlobalVars::stringMapper->ConvertBackFromISO(output);

					cout << output << endl;
				}
			}
			else if (task == "abbreviation-complete")
			{
				string str;
				while (std::getline(cin, str))
				{
    				if (encoding == "utf8")
				        str = GlobalVars::stringMapper->ConvertToISO(str);

					string output = GlobalVars::textCheckingServer->CompleteAbbreviatedText(str);

                    if (encoding == "utf8")
                        output =  GlobalVars::stringMapper->ConvertBackFromISO(output);

					cout << output << endl;
				}			
			}
			else if (task == "spellchecking")
			{
				string str;
				while (std::getline(cin, str))
				{
    				if (encoding == "utf8")
				        str = GlobalVars::stringMapper->ConvertToISO(str);

					string output = GlobalVars::textCheckingServer->SpellCheck_command_line_mode(str);

                    if (encoding == "utf8")
                        output =  GlobalVars::stringMapper->ConvertBackFromISO(output);

					cout << output << endl;
			    }
			}
			else if (task == "spellchecking-eval")
			{
				string str;
				while (std::getline(cin, str))
				{
    				if (encoding == "utf8")
				        str = GlobalVars::stringMapper->ConvertToISO(str);

					string output = GlobalVars::textCheckingServer->SpellChecking_evaluate(str);

                    if (encoding == "utf8")
                        output =  GlobalVars::stringMapper->ConvertBackFromISO(output);

					cout << output << endl;
			    }			
			}
			else if (task == "tokenize")
			{
				string text = "";
				string str;
				while (std::getline(cin, str))
				{
				
				    if (encoding == "utf8")
				    {
			        	str = GlobalVars::stringMapper->ConvertToISO(str);
			        }

    				if (text != "")
    					text += "\n";
					text += str;
			    }
			    
			    vector<string> sentences;
			    vector<string> delim_seqs;
				
				Splitter::SplitTextToSentences(text, sentences, delim_seqs);
				
				for (size_t i = 0; i < sentences.size(); i++)
				{
					if (i > 0)
					{
						for (size_t j = 0; j < delim_seqs[i].length(); j++)
						{
							if (delim_seqs[i][j] == '\n')
							{
								cout << endl;
								break;
							}
							
							if (j == delim_seqs[i].length() - 1)
								cout << " ";
						}
					}
						
					vector<string> tokens;
					vector<string> delim_seqs2;
					Splitter::SplitSentenceToTokens(sentences[i], tokens, delim_seqs2);
					
					for (size_t j = 0; j < tokens.size(); j++)
					{
						if (j > 0) cout << " ";
						
						if (encoding == "utf8")
						{
							cout << GlobalVars::stringMapper->ConvertBackFromISO(tokens[j]);
						}
						else
							cout << tokens[j];
					}
				}
			}

			

		return 0;
	}